Package: AccSamplingDesign
Title: Acceptance Sampling Plans Design
Version: 0.0.8
Authors@R: 
    c(person("Ha", "Truong", 
           email = "truongvietha87@gmail.com",
           role = c("aut", "cre", "cph")), 
      person("Victor", "Miranda", role = c("ths", "rev")),  
      person("Roger", "Kissling", role = c("ths", "rev"))  
    )
Description: Provides tools for designing and analyzing Acceptance Sampling plans.
    Supports both Attributes Sampling (Binomial and Poisson distributions) and 
    Variables Sampling (Normal and Beta distributions), enabling quality control 
    for fractional and compositional data. Uses nonlinear programming for sampling 
    plan optimization, minimizing sample size while controlling producer's and 
    consumer's risks. Operating Characteristic curves are available for plan visualization.
License: GPL-3
Encoding: UTF-8
Imports: stats, methods
Suggests: VGAM, knitr, rmarkdown
VignetteBuilder: knitr
URL: https://github.com/vietha/AccSamplingDesign
BugReports: https://github.com/vietha/AccSamplingDesign/issues
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-02-20 08:26:10 UTC; hatruong
Author: Ha Truong [aut, cre, cph],
  Victor Miranda [ths, rev],
  Roger Kissling [ths, rev]
Maintainer: Ha Truong <truongvietha87@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-20 10:30:26 UTC
Built: R 4.6.0; ; 2026-04-25 15:53:27 UTC; unix

Package: AccSamplingDesign
Title: Acceptance Sampling Plans Design
Version: 0.0.9
Authors@R: 
    c(person("Ha", "Truong", 
           email = "truongvietha87@gmail.com",
           role = c("aut", "cre", "cph")), 
      person("Victor", "Miranda", role = c("ths", "rev")),  
      person("Roger", "Kissling", role = c("ths", "rev"))  
    )
Description: Provides tools for designing and analyzing Acceptance Sampling plans.
    Supports both Attributes Sampling (Binomial and Poisson distributions) and 
    Variables Sampling (Normal and Beta distributions), enabling quality control 
    for fractional and compositional data. Uses nonlinear programming for sampling 
    plan optimization, minimizing sample size while controlling producer's and 
    consumer's risks. Operating Characteristic curves are available for plan visualization.
License: GPL-3
Encoding: UTF-8
Imports: stats, methods
Suggests: VGAM, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/vietha/AccSamplingDesign,
        https://doi.org/10.32614/RJ-2026-007
BugReports: https://github.com/vietha/AccSamplingDesign/issues
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-08-25 01:07:52 UTC; hatruong
Author: Ha Truong [aut, cre, cph],
  Victor Miranda [ths, rev],
  Roger Kissling [ths, rev]
Maintainer: Ha Truong <truongvietha87@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-25 07:00:18 UTC

export(accProb)
export(optAttrPlan)
export(optVarPlan)
export(optPlan)
export(muEst)
export(OCdata)
export(manualPlan)
S3method(as.list, OCdata)
S3method(as.data.frame, OCdata)
S3method(OCdata, AttrPlan)
S3method(OCdata, VarPlan)
S3method(accProb, AttrPlan)
S3method(accProb, VarPlan)
S3method(plot, AttrPlan)
S3method(plot, VarPlan)
S3method(summary, AttrPlan)
S3method(summary, VarPlan)
S3method(print, AttrPlan)
S3method(print, VarPlan)
S3method(print, OCdata)
S3method(summary, OCdata)
S3method(plot, OCdata)
# Importing necessary functions from other packages
importFrom("stats", "optim", "pbeta", "pbinom", "pnorm", "qnorm", "rbeta", "uniroot")
importFrom("stats", "pt")
importFrom("stats", "qbinom")
importFrom("stats", "ppois", "qpois")
importFrom("methods", "new")
importFrom("graphics", "abline", "grid")

# AccSamplingDesign <img src="https://cran.r-project.org/Rlogo.svg" align="right" height="100"/>

An R package for designing and analyzing **acceptance sampling plans**  
📦 Now available on
[CRAN](https://cran.r-project.org/package=AccSamplingDesign)! 🎉 —

See [NEWS.md](NEWS.md) for release notes and user-visible changes.

The package is described in Truong, Miranda, and Kissling (2026),
[*AccSamplingDesign: An R Package for Optimizing Acceptance Sampling
Plans*](https://doi.org/10.32614/RJ-2026-007), published in *The R Journal*.

## Overview

The **AccSamplingDesign** package provides flexible tools to create and
evaluate **acceptance sampling plans** in quality control, for both
attributes (pass/fail) and variables (measurable) data. It supports
optimization using nonlinear programming (NLP) to meet specified risks
while minimizing the required sample size.

### Key Features

- 🔍 **Attribute Sampling** (Binomial, Poisson): Decisions based on
  defect counts  
- 📈 **Variable Sampling** (Normal, Beta): Including **compositional
  data**  
- ⚙️ **Risk-Based Optimization**: Minimize sample size under `alpha` and
  `beta` constraints  
- 📊 **OC Curve Visualization**: Plot Operating Characteristic curves  
- 🔄 **Custom Plan Comparison**: Evaluate user-defined vs. optimized
  plans

------------------------------------------------------------------------

## Installation

``` r
# Install from CRAN
install.packages("AccSamplingDesign")

# Or install development version from GitHub
devtools::install_github("vietha/AccSamplingDesign")

# Load the package
library(AccSamplingDesign)
```

------------------------------------------------------------------------

## Examples

### 📌 Attribute Sampling (Binomial)

``` r
plan_attr <- optPlan(
  PRQ = 0.01,   # Acceptable quality
  CRQ = 0.05,   # Rejectable quality
  alpha = 0.02, # Producer's risk
  beta = 0.15,  # Consumer's risk
  distribution = "binomial"
)

summary(plan_attr)
accProb(plan_attr, 0.03)  # P(accept) if 3% defective
plot(plan_attr)           # OC curve
```

------------------------------------------------------------------------

### 📌 Variable Sampling (Normal, Known Sigma)

``` r
plan_var <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "known"
)

summary(plan_var)
plot(plan_var)
```

------------------------------------------------------------------------

### 📌 Variable Sampling (Normal, Unknown Sigma)

``` r
plan_var2 <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "unknown"
)

summary(plan_var2)
```

------------------------------------------------------------------------

### 📌 Variable Sampling (Beta, Known Theta)

``` r
plan_beta <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "known",
  LSL = 0.00001         # Lower Specification Limit
)

summary(plan_beta)
plot(plan_beta)              # By defect level
plot(plan_beta, by = "mean") # By mean value
```

------------------------------------------------------------------------

### 📌 Variable Sampling (Beta, Unknown Theta)

Unknown-theta plans support analytical Delta--MLE (`"delta_mle"`), analytical
Delta--MoM (`"delta_mom"`), and the earlier Govindaraju--Kissling sample-size
adjustment (`"gk_adjustment"`). Delta--MLE is used when `method` is omitted.

``` r
plan_beta <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "unknown",
  method = "delta_mle", # Default; may be omitted
  LSL = 0.00001
)

summary(plan_beta)
plot(plan_beta)              # By defect level
plot(plan_beta, by = "mean") # By mean value
```

------------------------------------------------------------------------

### 📌 Compare Custom vs. Optimal Plans

``` r
pd <- seq(0, 0.15, by = 0.001)

oc_opt <- OCdata(plan = plan_attr, pd = pd)

mplan1 <- manualPlan(n = plan_attr$n, c = plan_attr$c - 1, distribution = "binomial")
oc_alt1 <- OCdata(plan = mplan1, pd = pd)

plot(pd, oc_opt$paccept, type = "l", col = "blue", lwd = 2,
     xlab = "Proportion Defective", ylab = "Probability of Acceptance",
     main = "OC Curves Comparison for Attributes Sampling Plan")
lines(pd, oc_alt1$paccept, col = "red", lwd = 2, lty = 2)
legend("topright", legend = c("Optimal Plan", "Manual Plan (c - 1)"),
       col = c("blue", "red"), lty = c(1, 2), lwd = 2)
```

------------------------------------------------------------------------

# Additional Notes

This README provides a quick start for using the **AccSamplingDesign**
package. For a full discussion of the statistical foundations, models,
and optimization methods used, please refer to the foundation sources
such as:

- Schilling, E.G., & Neubauer, D.V. (2017). *Acceptance Sampling in
  Quality Control* (3rd ed.). CRC Press.  
- Wilrich, P.T. (2004). In *Frontiers in Statistical Quality Control
  7*.  
- Govindaraju, K., & Kissling, R. (2015). *Quality Engineering*, 27(1),
  1–13.

------------------------------------------------------------------------

## Contributing

Contributions, suggestions, and bug reports are welcome!  
Please use [GitHub
Issues](https://github.com/vietha/AccSamplingDesign/issues) or submit a
pull request.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(AccSamplingDesign)
library(VGAM)

## -----------------------------------------------------------------------------
# Create an attribute plan with binomial assumption
plan_attr <- optPlan(
  PRQ = 0.01,   # Acceptable quality level (1%)
  CRQ = 0.05,   # Rejectable quality level (5%)
  alpha = 0.02, # Producer's risk
  beta = 0.15,  # Consumer's risk
  distribution = "binomial"
)

# Summary of the plan
summary(plan_attr)

# Probability of accepting 3% defective
accProb(plan_attr, 0.03)

# Plot the OC curve
plot(plan_attr)

## -----------------------------------------------------------------------------
# Create a variable plan assuming known sigma
plan_var <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "known"
)

# Summary
summary(plan_var)

# Plot OC curve
plot(plan_var)

## -----------------------------------------------------------------------------
# Create a variable plan assuming known sigma
plan_var2 <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "unknown"
)

# Summary
summary(plan_var2)

## -----------------------------------------------------------------------------
# Create a variable plan using Beta distribution
plan_beta <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "known",
  LSL = 0.00001          # Lower Specification Limit
)

# Summary
summary(plan_beta)

# Plot OC curve
plot(plan_beta)
# Plot OC curve be the process mean
plot(plan_beta, by = "mean")

## -----------------------------------------------------------------------------
# Create a variable plan using Beta distribution
plan_beta2 <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "unknown",
  LSL = 0.00001
)

# Summary
summary(plan_beta2)

## -----------------------------------------------------------------------------
plan_beta2$sample_size  # integer sample size used in plan
plan_beta2$n            # raw computed (non-rounded) value

## -----------------------------------------------------------------------------
# Example data
y <- c(0.75, 0.68, 0.72, 0.70, 0.76)

# Fit Beta model
fit <- vglm(y ~ 1, betaff, data = data.frame(y = y))

# Extract estimated parameters
coef(fit, matrix = TRUE)

## -----------------------------------------------------------------------------
theta_est <- exp(coef(fit, matrix = TRUE)[1, "loglink(phi)"])
theta_est
optPlan(PRQ = 0.05, CRQ = 0.20, alpha = 0.05, beta = 0.10,
           USL = 0.8, distribution = "beta", theta_type = "unknown",
           theta = theta_est)

## -----------------------------------------------------------------------------
# Define range of defect rates
pd <- seq(0, 0.15, by = 0.001)

# Generate OC data from optimal plan
oc_opt <- OCdata(plan = plan_attr, pd = pd)

# Compare with manual plans
mplan1 <- manualPlan(n = plan_attr$n, c = plan_attr$c - 1, distribution = "binomial")
oc_alt1 <- OCdata(plan = mplan1, pd = pd)

# Plot comparison
plot(pd, oc_opt$paccept, type = "l", col = "blue", lwd = 2,
     xlab = "Proportion Defective", ylab = "Probability of Acceptance",
     main = "OC Curves Comparison for Attributes Sampling Plan")
lines(pd, oc_alt1$paccept, col = "red", lwd = 2, lty = 2)
legend("topright", legend = c("Optimal Plan", "Manual Plan c - 1"),
       col = c("blue", "red"), lty = c(1, 2), lwd = 2)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(AccSamplingDesign)
library(VGAM)

## -----------------------------------------------------------------------------
# Create an attribute plan with binomial assumption
plan_attr <- optPlan(
  PRQ = 0.01,   # Acceptable quality level (1%)
  CRQ = 0.05,   # Rejectable quality level (5%)
  alpha = 0.02, # Producer's risk
  beta = 0.15,  # Consumer's risk
  distribution = "binomial"
)

# Summary of the plan
summary(plan_attr)

# Probability of accepting 3% defective
accProb(plan_attr, 0.03)

# Plot the OC curve
plot(plan_attr)

## -----------------------------------------------------------------------------
# Create a variable plan assuming known sigma
plan_var <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "known"
)

# Summary
summary(plan_var)

# Plot OC curve
plot(plan_var)

## -----------------------------------------------------------------------------
# Create a variable plan assuming known sigma
plan_var2 <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "unknown"
)

# Summary
summary(plan_var2)

## -----------------------------------------------------------------------------
# Create a variable plan using Beta distribution
plan_beta <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "known",
  LSL = 0.00001          # Lower Specification Limit
)

# Summary
summary(plan_beta)

# Plot OC curve
plot(plan_beta)
# Plot OC curve be the process mean
plot(plan_beta, by = "mean")

## -----------------------------------------------------------------------------
# Create a variable plan using Beta distribution
plan_beta2 <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "unknown",
  method = "delta_mle",
  LSL = 0.00001
)

# Summary
summary(plan_beta2)

## -----------------------------------------------------------------------------
plan_beta2$sample_size  # integer sample size used in plan
plan_beta2$n            # raw computed (non-rounded) value

## -----------------------------------------------------------------------------
# Example data
y <- c(0.75, 0.68, 0.72, 0.70, 0.76)

# Fit Beta model
fit <- vglm(y ~ 1, betaff, data = data.frame(y = y))

# Extract estimated parameters
coef(fit, matrix = TRUE)

## -----------------------------------------------------------------------------
theta_est <- exp(coef(fit, matrix = TRUE)[1, "loglink(phi)"])
theta_est
optPlan(PRQ = 0.05, CRQ = 0.20, alpha = 0.05, beta = 0.10,
           USL = 0.8, distribution = "beta", theta_type = "unknown",
           theta = theta_est)

## -----------------------------------------------------------------------------
# Define range of defect rates
pd <- seq(0, 0.15, by = 0.001)

# Generate OC data from optimal plan
oc_opt <- OCdata(plan = plan_attr, pd = pd)

# Compare with manual plans
mplan1 <- manualPlan(n = plan_attr$n, c = plan_attr$c - 1, distribution = "binomial")
oc_alt1 <- OCdata(plan = mplan1, pd = pd)

# Plot comparison
plot(pd, oc_opt$paccept, type = "l", col = "blue", lwd = 2,
     xlab = "Proportion Defective", ylab = "Probability of Acceptance",
     main = "OC Curves Comparison for Attributes Sampling Plan")
lines(pd, oc_alt1$paccept, col = "red", lwd = 2, lty = 2)
legend("topright", legend = c("Optimal Plan", "Manual Plan c - 1"),
       col = c("blue", "red"), lty = c(1, 2), lwd = 2)

## -----------------------------------------------------------------------------
## summary.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: Summary methods for acceptance sampling plans
##
## Changelogs:
## -----------------------------------------------------------------------------


# S3 methods for AttrPlan
#' @export
print.AttrPlan <- function(x, ...) {
  cat("AttrPlan object:\n")
  cat(" Distribution:", x$distribution, "\n")
  cat(" Sample size (n):", x$n, "\n")
  if (length(x$c) > 0) cat(" Acceptance number (c):", x$c, "\n")
  cat("\nNOTE: summary(plan) for detail report,",
      "\n      plot(plan) for quick OC visualization,",
      "\n      OCdata(plan) to extract data for evaluation and custom plots.\n")
}

#' @export
summary.AttrPlan <- function(object, ...) {
  cat("Attributes Acceptance Sampling Plan: \n")
  cat(" Distribution:", object$distribution, "\n")
  cat(" Sample Size (n):", object$n, "\n")
  cat(" Acceptance Number (c):", object$c, "\n")
  cat(" Producer's Risk (PR =", object$PR, ") at PRQ =", object$PRQ, "\n")
  cat(" Consumer's Risk (CR =", object$CR, ") at CRQ =", object$CRQ, "\n")
  if(object$measurement_error > 0) {
    cat(" Measurement Error:", object$measurement_error, "\n")
  }
  invisible(object)
}


# S3 methods for VarPlan
#' @export
print.VarPlan <- function(x, ...) {
  cat("VarPlan object:\n")
  cat(" Distribution:", x$distribution, "\n")
  if(x$distribution == "normal") {
    cat(" Sigma type:", x$sigma_type, "\n")
  } else {
    cat(" Theta type:", x$theta_type, "\n")
    if (identical(x$theta_type, "unknown")) {
      method <- if (is.null(x$method)) "delta_mle" else x$method
      cat(" Unknown-theta method:", .beta_theta_method_label(method), "\n")
    }
  }
  cat(" Sample size (n):", x$sample_size, "\n")
  if (length(x$k) > 0) cat(" Acceptability constant (k):", x$k, "\n")
  cat("\nNOTE: summary(plan) for detail report,",
      "\n      plot(plan) for quick OC visualization,",
      "\n      OCdata(plan) to extract data for evaluation and custom plots.\n")
}

#' @export
summary.VarPlan <- function(object, ...) {
  cat("Variables Acceptance Sampling Plan\n")
  cat(" Distribution:", object$distribution, "\n")
  
  #cat(" Sample Size (n):", object$sample_size, "\n")
  # Show rounded sample size and raw n
  if (!is.null(object$n) && object$n != object$sample_size) {
    cat(" Sample Size (n, rounded up):", object$sample_size, 
        " [raw n =", object$n, "]\n")
  } else {
    cat(" Sample Size (n):", object$sample_size, "\n")
  }
  
  cat(" Acceptability Constant (k):", round(object$k, 3), "\n")
  if(object$distribution == "normal") {
    cat(" Population Standard Deviation:", object$sigma_type, "\n")
  } else {
    cat(" Population Precision Parameter (theta):", object$theta_type, "\n")
    if (identical(object$theta_type, "unknown")) {
      method <- if (is.null(object$method)) "delta_mle" else object$method
      cat(" Unknown-theta method:", .beta_theta_method_label(method), "\n")
    }
  }
  cat(" Producer's Risk (PR =", object$PR, ") at PRQ =", object$PRQ, "\n")
  cat(" Consumer's Risk (CR =", object$CR, ") at CRQ =", object$CRQ, "\n")
  if (!is.null(object$LSL)) {
    cat(" Lower Specification Limit (LSL):", object$LSL, "\n")
  }
  if (!is.null(object$USL)) {
    cat(" Upper Specification Limit (USL):", object$USL, "\n")
  }
  if(!is.null(object$measurement_error)) {
    cat(" Measurement Error:", object$measurement_error, "\n")
  }
  invisible(object)
}

## -----------------------------------------------------------------------------
## optPlan.R --- 
##
## Author: Ha Truong
##
## Created: 22 Apr 2025
##
## Purposes: Design variable acceptance sampling plans (normal/beta)
##
## Changelogs:
## -----------------------------------------------------------------------------

#' Optimal Acceptance Sampling Plan
#' @export
optPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10, USL = NULL, LSL = NULL,
                    distribution = c("binomial", "poisson", "normal", "beta"),
                    sigma_type = c("known", "unknown"),
                    theta_type = c("known", "unknown"),
                    sigma = NULL, theta = NULL,
                    method = c("delta_mle", "delta_mom", "gk_adjustment")) {
  method_missing <- missing(method)
  
  # Match arguments to ensure valid input
  distribution <- match.arg(distribution)
  sigma_type <- match.arg(sigma_type)
  theta_type <- match.arg(theta_type)
  method <- .normalize_beta_theta_method(
    method = if (method_missing) NULL else method,
    distribution = distribution,
    theta_type = theta_type,
    method_missing = method_missing
  )
  
  # Ensure PRQ, CRQ, alpha, and beta are within valid ranges based on distribution
  check_quality <- function(q, distribution) {
    if (distribution == "normal" && (q < 0 || q > 1)) {
      return(FALSE)
    }
    if (distribution == "beta" && (q <= 0 || q >= 1)) {
      return(FALSE)
    }
    if (distribution == "binomial" && (q <= 0 || q >= 1)) {
      return(FALSE)
    }
    return(TRUE)
  }
  
  # Ensure PRQ and CRQ are provided
  if (missing(PRQ) || missing(CRQ)) {
    stop("Producer and Consumer Risk Points must be provided.")
  }
  
  # Ensure Consumer Risk Quality (CRQ) > Producer Risk Quality (PRQ)
  if (CRQ <= PRQ) {
    stop("Consumer Risk Quality (CRQ) must be greater than Producer Risk Quality (PRQ).")
  }
  
  if (!check_quality(PRQ, distribution) || !check_quality(CRQ, distribution)) {
    stop("PRQ and CRQ are out of bounds for the selected distribution.")
  }
  
  if (alpha <= 0 || alpha >= 1 || beta <= 0 || beta >= 1) {
    stop("alpha and beta must be between 0 and 1 (exclusive).")
  }
  
  # Set default if not provided
  if (is.null(sigma_type)) sigma_type <- "known"
  if (is.null(theta_type)) theta_type <- "known"
  
  if (!is.null(USL) && !is.null(LSL)) {
    stop("Double specification limits (both USL and LSL) are not supported. 
         Please specify only one limit: either USL or LSL.")
  }
  # This upper limit case
  if (!is.null(USL)) {
    limit_type <- "upper"
    spec_limit <- USL
  } else if (!is.null(LSL)) { # This lower limit case
    limit_type <- "lower"
    spec_limit <- LSL
  } else {
    limit_type <- "upper" # set this as default
  }
  
  # Additional checks for beta distribution
  if (distribution == "beta") {
    if(is.null(USL) && is.null(LSL)){
      stop("For the beta distribution, a specification limit must be provided.")
    }
    if (is.null(theta)) {
      stop("For the beta distribution, theta must be provided.")
    }
  }
  
  # Check measurement error is non-negative
  # if (measurement_error < 0) {
  #   stop("measurement_error must be non-negative.")
  # }
  
  # Init an optimal plan
  opt_plan = NULL
  
  # ------------Binomial ----------
  if (distribution == "binomial"|| distribution == "poisson") {
    opt_plan <- optAttrPlan(PRQ = PRQ, CRQ = CRQ, alpha = alpha, beta = beta, 
                            distribution = distribution)
  }
  # ------------Normal/Beta ------------
  if (distribution == "normal") {
    opt_plan <- optVarPlan(PRQ = PRQ, CRQ = CRQ, alpha = alpha, beta = beta, 
                           USL = USL, LSL = LSL, distribution = distribution,
                           sigma_type = sigma_type, theta_type = theta_type,
                           sigma = sigma, theta = theta)
  }
  if (distribution == "beta") {
    if (theta_type == "unknown") {
      opt_plan <- optVarPlan(
        PRQ = PRQ, CRQ = CRQ, alpha = alpha, beta = beta,
        USL = USL, LSL = LSL, distribution = distribution,
        sigma_type = sigma_type, theta_type = theta_type,
        sigma = sigma, theta = theta, method = method
      )
    } else {
      opt_plan <- optVarPlan(
        PRQ = PRQ, CRQ = CRQ, alpha = alpha, beta = beta,
        USL = USL, LSL = LSL, distribution = distribution,
        sigma_type = sigma_type, theta_type = theta_type,
        sigma = sigma, theta = theta
      )
    }
  }
  return(opt_plan)
}

## -----------------------------------------------------------------------------
## test-optPlan.R
##
## Author: Ha Truong
## Created: 12 Nov 2025
## Purpose: Test optPlan wrapper functionality
## -----------------------------------------------------------------------------

test_that("optPlan works for binomial distribution", {
  plan <- optPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  
  expect_true(is.list(plan))
  expect_true(all(c("n", "c") %in% names(plan)))
  expect_gt(plan$n, 0)
  expect_gte(plan$c, 0)
})

test_that("optPlan works for normal, known sigma", {
  plan <- optPlan(PRQ = 0.005, CRQ = 0.03,
                  alpha = 0.05, beta = 0.10, 
                  distribution = "normal", sigma_type = "known")
  
  expect_true(is.list(plan))
  expect_gt(plan$n, 0)
  expect_gt(plan$k, 0)
})

test_that("optPlan works for beta distribution", {
  plan <- optPlan(PRQ = 0.2, CRQ = 0.3, USL = 0.5,
                  distribution = "beta", theta = 0.2)
  
  expect_true(is.list(plan))
})


# Error behavior tests
# -----------------------------------------------------------------------------

test_that("optPlan errors when CRQ <= PRQ", {
  expect_error(
    optPlan(PRQ = 0.05, CRQ = 0.05, distribution = "binomial"),
    "must be greater"
  )
})

test_that("optPlan errors when PRQ or CRQ are missing", {
  expect_error(optPlan(CRQ = 0.05), "must be provided")
  expect_error(optPlan(PRQ = 0.01), "must be provided")
})

test_that("invalid alpha/beta rejected", {
  expect_error(optPlan(PRQ = 0.01, CRQ = 0.05, alpha = -1))
  expect_error(optPlan(PRQ = 0.01, CRQ = 0.05, beta = 1.1))
})

test_that("specifying both USL and LSL errors", {
  expect_error(
    optPlan(PRQ = 0.01, CRQ = 0.05,
            USL = 1, LSL = 0, distribution = "normal"),
    "only one limit"
  )
})

test_that("beta requires theta and one spec limit", {
  expect_error(
    optPlan(PRQ = 0.1, CRQ = 0.2, distribution = "beta"),
    "must be provided"
  )
  expect_error(
    optPlan(PRQ = 0.1, CRQ = 0.2, distribution = "beta", theta = 0.1),
    "specification"
  )
})

# Consistency test (delegation)
# -----------------------------------------------------------------------------

test_that("optPlan delegates to optAttrPlan for binomial", {
  plan1 <- optPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  plan2 <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  
  expect_equal(plan1, plan2)
})

test_that("optPlan delegates to optVarPlan for normal", {
  plan1 <- optPlan(PRQ = 0.01, CRQ = 0.05, distribution = "normal")
  plan2 <- optVarPlan(PRQ = 0.01, CRQ = 0.05, distribution = "normal")
  
  expect_equal(plan1, plan2)
})

test_that("optPlan forwards the unknown-theta method", {
  arguments <- list(
    PRQ = 0.01, CRQ = 0.05, distribution = "beta",
    theta_type = "unknown", theta = 300, USL = 0.05,
    method = "delta_mom"
  )
  expect_equal(do.call(optPlan, arguments), do.call(optVarPlan, arguments))
  expect_identical(do.call(optPlan, arguments)$method, "delta_mom")
})

test_that("optPlan defaults unknown-theta Beta plans to Delta-MLE", {
  plan <- optPlan(
    PRQ = 0.01, CRQ = 0.05, distribution = "beta",
    theta_type = "unknown", theta = 300, USL = 0.05
  )
  expect_identical(plan$method, "delta_mle")
})

test_that("optPlan rejects method when it is not applicable", {
  expect_error(
    optPlan(0.01, 0.05, distribution = "normal", method = "delta_mle"),
    "only applicable"
  )
  expect_error(
    optPlan(0.01, 0.05, distribution = "binomial", method = "delta_mle"),
    "only applicable"
  )
  expect_error(
    optPlan(0.01, 0.05, distribution = "beta", theta = 300, USL = 0.05,
            method = "delta_mle"),
    "only applicable"
  )
})

# Edge cases
# -----------------------------------------------------------------------------

test_that("optPlan sets default limit_type to upper when no limits provided", {
  plan <- optPlan(PRQ = 0.01, CRQ = 0.05, distribution = "normal")
  expect_true("n" %in% names(plan))
})

## -----------------------------------------------------------------------------
## OCdata.R --- 
##
## Author: Ha Truong
##
## Created: 11 Mar 2025
##
## Purposes: Generate OC curve data
##
## Changelogs:
## -----------------------------------------------------------------------------
#' @export
OCdata <- function(plan, pd = NULL) {
  UseMethod("OCdata")
}

#' @export
OCdata.AttrPlan <- function(plan, pd = NULL) {
  if (is.null(pd)) {
    pd <- seq(1e-10, min(plan$CRQ * 2, 1), length.out = 100)
  }
  paccept <- sapply(pd, function(p) accProb(plan, p))
  
  structure(list(
    pd = pd,
    paccept = paccept,
    process_means = numeric(0),
    dist = plan$distribution,
    n = plan$sample_size,
    c = plan$c#,
    #k = numeric(0)
  ), class = "OCdata")
}

#' @export
OCdata.VarPlan <- function(plan, pd = NULL) {
  if (is.null(pd)) {
    pd <- seq(1e-10, min(plan$CRQ * 2, 1), length.out = 100)
  }
  paccept <- sapply(pd, function(p) accProb(plan, p))
  
  mean_level <- NULL
  if (!is.null(plan$USL) || !is.null(plan$LSL)) {
    mean_level <- sapply(pd, function(p) muEst(
      p, USL = plan$USL, LSL = plan$LSL,
      sigma = plan$sigma,
      theta = plan$theta,
      dist = plan$distribution
    ))
  }
  
  structure(list(
    pd = pd,
    paccept = paccept,
    process_means = mean_level,
    dist = plan$distribution,
    n = plan$sample_size,
    #c = numeric(0),
    k = plan$k
  ), class = "OCdata")
}

# S3 methods for OCdata
#' @export
print.OCdata <- function(x, ...) {
  cat("OCdata object:\n")
  cat(" Distribution:", x$dist, "\n")
  cat(" Sample size (n):", x$n, "\n")
  if (length(x$k) > 0) cat(" Acceptability constant (k):", x$k, "\n")
  if (length(x$c) > 0) cat(" Acceptance number (c):", x$c, "\n")
}

#' @export
summary.OCdata <- function(object, ...) {
  cat("Summary of OCdata:\n")
  print(object)
  cat(" # of pd values:", length(object$pd), "\n")
  cat(" # of P(accept) values:", length(object$paccept), "\n")
  if (length(object$process_means) > 0) {
    cat(" Process means available (length:", length(object$process_means), ")\n")
  }
}

#' @export
plot.OCdata <- function(x, by = c("pd", "mean"), ...) {
  by <- match.arg(by)
  
  if (by == "pd") {
    plot(x$pd, x$paccept, type = "l", col = "red", lwd = 2,
         main = "OC Curve by Proportion Nonconforming", 
         xlab = "Proportion Nonconforming", ylab = "P(accept)", ...)
    grid()
  } else {
    if (length(x$process_means) > 0) {
      plot(x$process_means, x$paccept, type = "l", col = "blue", lwd = 2,
           main = "OC Curve by Mean Levels", xlab = "Mean Level", ylab = "P(accept)", ...)
      grid()
    } else {
      message("Mean-level plot not available.")
    }
  }
}

#' @export
as.list.OCdata <- function(x, ...) {
  stopifnot(inherits(x, "OCdata"))
  unclass(x)
}

#' @export
as.data.frame.OCdata <- function(x, row.names = NULL, optional = FALSE, ...) {
  stopifnot(inherits(x, "OCdata"))
  
  df <- data.frame(
    pd = x$pd,
    paccept = x$paccept,
    stringsAsFactors = FALSE
  )
  
  if (!is.null(x$process_means) && length(x$process_means) == length(x$pd)) {
    df$process_means <- x$process_means
  }
  
  df
}

#' 
#' #' @export
#' OCdata.paccept <- function(x) {
#'   stopifnot(inherits(x, "OCdata"))
#'   x$paccept
#' }
#' 
#' #' @export
#' OCdata.process_means <- function(x) {
#'   stopifnot(inherits(x, "OCdata"))
#'   x$process_means
#' }

## -----------------------------------------------------------------------------
## test-OCdata.R
##
## Author: Ha Truong
## Created: 12 Nov 2025
## Purpose: Test OCdata functions
## -----------------------------------------------------------------------------



# Attributes Plans
## -----------------------------------------------------------------------------
test_that("OCdata.AttrPlan works for binomial", {
  plan_attr <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  oc <- OCdata.AttrPlan(plan_attr)
  
  expect_s3_class(oc, "OCdata")
  expect_equal(oc$dist, "binomial")
  expect_length(oc$pd, 100)
  expect_length(oc$paccept, 100)
  
  # Custom pd
  custom_pd <- seq(0, 0.1, length.out = 5)
  oc2 <- OCdata.AttrPlan(plan_attr, pd = custom_pd)
  expect_equal(oc2$pd, custom_pd)
  expect_length(oc2$paccept, length(custom_pd))
})

test_that("OCdata.AttrPlan works for poisson", {
  plan_attr <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "poisson")
  oc <- OCdata.AttrPlan(plan_attr)
  
  expect_s3_class(oc, "OCdata")
  expect_equal(oc$dist, "poisson")
})

test_that("OCdata.AttrPlan creates valid object", {
  plan_attr <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  oc <- OCdata.AttrPlan(plan_attr)
  expect_s3_class(oc, "OCdata")
  expect_length(oc$pd, 100)
  expect_length(oc$paccept, 100)
})


# Variables Plans
## -----------------------------------------------------------------------------

test_that("OCdata.VarPlan creates valid object", {
  plan_var <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                         distribution = "normal", sigma_type = "known", sigma = 1)
  oc <- OCdata.VarPlan(plan_var)
  expect_s3_class(oc, "OCdata")
  expect_length(oc$paccept, 100)
})

test_that("OCdata.VarPlan works for normal, known sigma", {
  plan_var <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                         distribution = "normal", sigma_type = "known", sigma = 1)
  oc <- OCdata.VarPlan(plan_var)
  
  expect_s3_class(oc, "OCdata")
  expect_equal(oc$dist, "normal")
  expect_length(oc$paccept, length(oc$pd))
})

test_that("OCdata.VarPlan works for normal, unknown sigma", {
  plan_var <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                         distribution = "normal", sigma_type = "unknown")
  oc <- OCdata.VarPlan(plan_var)
  
  expect_s3_class(oc, "OCdata")
})

test_that("OCdata.VarPlan works for beta, upper and lower limits", {
  plan_beta_upper <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                                distribution = "beta", theta_type = "known",
                                theta = 100, USL = 0.1)
  oc_upper <- OCdata.VarPlan(plan_beta_upper)
  expect_s3_class(oc_upper, "OCdata")
  expect_length(oc_upper$paccept, 100)
  
  plan_beta_lower <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                                distribution = "beta", theta_type = "known",
                                theta = 100, LSL = 0.1)
  oc_lower <- OCdata.VarPlan(plan_beta_lower)
  expect_s3_class(oc_lower, "OCdata")
})

test_that("OCdata uses each unknown-theta method stored in a Beta plan", {
  pd <- c(0.01, 0.03, 0.05)
  for (method in c("delta_mle", "delta_mom", "gk_adjustment")) {
    for (limit_name in c("USL", "LSL")) {
      arguments <- list(
        distribution = "beta", n = 45, k = 2.2,
        theta = 300, theta_type = "unknown", method = method
      )
      arguments[[limit_name]] <- 0.05
      plan <- do.call(manualPlan, arguments)
      oc <- OCdata(plan, pd = pd)

      expect_s3_class(oc, "OCdata")
      expect_equal(oc$paccept, vapply(pd, function(p) accProb(plan, p), numeric(1)))
      expect_true(all(is.finite(oc$paccept)))
    }
  }
})

# S3 methods
## -----------------------------------------------------------------------------

test_that("print.OCdata and summary.OCdata work", {
  plan_attr <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  oc <- OCdata.AttrPlan(plan_attr)
  
  expect_output(print(oc), "OCdata object")
  expect_output(summary(oc), "Summary of OCdata")
})

test_that("plot.OCdata works", {
  plan_var <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                         distribution = "normal", sigma_type = "known", sigma = 1)
  oc <- OCdata.VarPlan(plan_var)
  
  expect_silent(plot(oc, by = "pd"))
  expect_message(plot(oc, by = "mean"))
  
  plan_attr <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  oc_attr <- OCdata.AttrPlan(plan_attr)
  expect_message(plot(oc_attr, by = "mean"), "not available")
})

## Internal methods for Beta variables plans with an estimated precision.
##
## These helpers are package-native implementations of the analytical methods.
## They intentionally have no dependency on research scripts, reports, or data.

.beta_theta_methods <- c("delta_mle", "delta_mom", "gk_adjustment")

.beta_theta_method_label <- function(method) {
  labels <- c(
    delta_mle = "Delta-MLE",
    delta_mom = "Delta-MoM",
    gk_adjustment = "Govindaraju-Kissling adjustment"
  )
  method <- match.arg(method, names(labels))
  unname(labels[[method]])
}

.normalize_beta_theta_method <- function(method, distribution, theta_type,
                                         method_missing = FALSE) {
  applicable <- identical(distribution, "beta") &&
    identical(theta_type, "unknown")

  if (!applicable) {
    if (!method_missing) {
      stop(
        "method is only applicable when distribution = \"beta\" and ",
        "theta_type = \"unknown\".",
        call. = FALSE
      )
    }
    return(NULL)
  }

  if (method_missing || is.null(method)) {
    return(.beta_theta_methods[[1L]])
  }

  match.arg(method, .beta_theta_methods)
}

.validate_beta_parameters <- function(mu, theta) {
  if (length(mu) != 1L || !is.finite(mu) || mu <= 0 || mu >= 1) {
    stop("mu must be one finite value strictly between 0 and 1.", call. = FALSE)
  }
  if (length(theta) != 1L || !is.finite(theta) || theta <= 0) {
    stop("theta must be one finite positive value.", call. = FALSE)
  }
}

.beta_raw_moments <- function(mu, theta, max_order = 4L) {
  .validate_beta_parameters(mu, theta)
  if (length(max_order) != 1L || max_order < 1L || max_order != as.integer(max_order)) {
    stop("max_order must be a positive integer.", call. = FALSE)
  }

  alpha <- mu * theta
  vapply(seq_len(max_order), function(order) {
    offsets <- seq.int(0, order - 1L)
    prod((alpha + offsets) / (theta + offsets))
  }, numeric(1L))
}

.beta_mom_covariance <- function(mu, theta) {
  # Return the per-observation asymptotic covariance Sigma for (mu_hat,
  # theta_hat). The covariance for a sample of size n is Sigma / n.
  moments <- .beta_raw_moments(mu, theta, max_order = 4L)
  m1 <- moments[[1L]]
  m2 <- moments[[2L]]
  variance <- m2 - m1^2

  moment_covariance <- matrix(
    c(
      variance,
      moments[[3L]] - m1 * m2,
      moments[[3L]] - m1 * m2,
      moments[[4L]] - m2^2
    ),
    nrow = 2L,
    byrow = TRUE
  )

  # Jacobian of (mu_hat, theta_hat) with respect to the first two raw
  # moments. Keeping the off-diagonal covariance is important near the
  # boundaries of the Beta distribution.
  dtheta_dm1 <- ((1 - 2 * m1) * variance +
    2 * m1^2 * (1 - m1)) / variance^2
  dtheta_dm2 <- -m1 * (1 - m1) / variance^2
  jacobian <- matrix(
    c(1, 0, dtheta_dm1, dtheta_dm2),
    nrow = 2L,
    byrow = TRUE
  )

  covariance <- jacobian %*% moment_covariance %*% t(jacobian)
  (covariance + t(covariance)) / 2
}

.beta_mle_information <- function(mu, theta) {
  .validate_beta_parameters(mu, theta)

  alpha_trigamma <- trigamma(mu * theta)
  beta_trigamma <- trigamma((1 - mu) * theta)
  theta_trigamma <- trigamma(theta)

  # Expected information for one observation, parameterized directly by
  # (mu, theta). This ordering must agree with the decision-rule gradient.
  information <- matrix(
    c(
      theta^2 * (alpha_trigamma + beta_trigamma),
      theta * (mu * alpha_trigamma - (1 - mu) * beta_trigamma),
      theta * (mu * alpha_trigamma - (1 - mu) * beta_trigamma),
      mu^2 * alpha_trigamma +
        (1 - mu)^2 * beta_trigamma - theta_trigamma
    ),
    nrow = 2L,
    byrow = TRUE
  )

  if (any(!is.finite(information))) {
    stop("Beta MLE Fisher information is not finite.", call. = FALSE)
  }
  information
}

.beta_mle_covariance <- function(mu, theta) {
  .validate_beta_parameters(mu, theta)
  alpha <- mu * theta
  beta_shape <- (1 - mu) * theta

  # Inverting in the conventional (alpha, beta) parameterization avoids the
  # severe scale imbalance of direct (mu, theta) information when theta is
  # large. The Jacobian then maps the covariance back to (mu, theta).
  shape_information <- matrix(
    c(
      trigamma(alpha) - trigamma(theta), -trigamma(theta),
      -trigamma(theta), trigamma(beta_shape) - trigamma(theta)
    ),
    nrow = 2L,
    byrow = TRUE
  )
  shape_covariance <- tryCatch(
    solve(shape_information),
    error = function(error) {
      stop(
        "Beta MLE Fisher information is singular: ", conditionMessage(error),
        call. = FALSE
      )
    }
  )
  transformation <- matrix(
    c((1 - mu) / theta, -mu / theta, 1, 1),
    nrow = 2L,
    byrow = TRUE
  )
  covariance <- transformation %*% shape_covariance %*% t(transformation)

  if (any(!is.finite(covariance))) {
    stop("Beta MLE covariance is not finite.", call. = FALSE)
  }
  # Return per-observation covariance; callers divide by sample size.
  (covariance + t(covariance)) / 2
}

.beta_delta_statistic <- function(mu, theta, k, limit_type) {
  .validate_beta_parameters(mu, theta)
  limit_type <- match.arg(limit_type, c("upper", "lower"))
  if (length(k) != 1L || !is.finite(k) || k < 0) {
    stop("k must be one finite non-negative value.", call. = FALSE)
  }

  direction <- if (limit_type == "upper") 1 else -1
  standard_deviation <- sqrt(mu * (1 - mu) / (theta + 1))
  mu + direction * k * standard_deviation
}

.beta_delta_gradient <- function(mu, theta, k, limit_type) {
  .validate_beta_parameters(mu, theta)
  limit_type <- match.arg(limit_type, c("upper", "lower"))
  if (length(k) != 1L || !is.finite(k) || k < 0) {
    stop("k must be one finite non-negative value.", call. = FALSE)
  }

  # The sign changes both k-dependent derivatives for a lower limit. The
  # parameter order is (mu, theta), matching both covariance implementations.
  direction <- if (limit_type == "upper") 1 else -1
  c(
    mu = 1 + direction * k * (1 - 2 * mu) /
      (2 * sqrt((theta + 1) * mu * (1 - mu))),
    theta = -direction * k * sqrt(mu * (1 - mu)) /
      (2 * (theta + 1)^(3 / 2))
  )
}

.beta_delta_acceptance_probability <- function(mu, theta, n, k, limit,
                                               limit_type, method) {
  .validate_beta_parameters(mu, theta)
  limit_type <- match.arg(limit_type, c("upper", "lower"))
  method <- match.arg(method, c("delta_mle", "delta_mom"))
  if (length(n) != 1L || !is.finite(n) || n <= 0) {
    stop("n must be one finite positive value.", call. = FALSE)
  }
  if (length(limit) != 1L || !is.finite(limit) || limit <= 0 || limit >= 1) {
    stop("The Beta specification limit must be strictly between 0 and 1.",
         call. = FALSE)
  }

  covariance <- if (method == "delta_mle") {
    .beta_mle_covariance(mu, theta)
  } else {
    .beta_mom_covariance(mu, theta)
  }
  gradient <- .beta_delta_gradient(mu, theta, k, limit_type)
  statistic <- .beta_delta_statistic(mu, theta, k, limit_type)

  # Covariance helpers return per-observation Sigma. Divide the quadratic form
  # by n exactly once to obtain the finite-sample Delta variance.
  delta_variance <- drop(t(gradient) %*% covariance %*% gradient) / n
  if (!is.finite(delta_variance) || delta_variance <= 0) {
    stop("Delta-method variance must be finite and positive.", call. = FALSE)
  }

  z_value <- if (limit_type == "upper") {
    (limit - statistic) / sqrt(delta_variance)
  } else {
    (statistic - limit) / sqrt(delta_variance)
  }
  probability <- pnorm(z_value)
  if (length(probability) != 1L || !is.finite(probability) ||
      probability < 0 || probability > 1) {
    stop("Delta acceptance probability is not finite or is outside [0, 1].",
         call. = FALSE)
  }
  probability
}

test_that("unknown-theta method validation recognizes supported methods", {
  normalize <- AccSamplingDesign:::.normalize_beta_theta_method

  expect_identical(
    normalize(NULL, "beta", "unknown", method_missing = TRUE),
    "delta_mle"
  )
  expect_identical(
    normalize("delta_mom", "beta", "unknown"),
    "delta_mom"
  )
  expect_identical(
    normalize("gk_adjustment", "beta", "unknown"),
    "gk_adjustment"
  )
  expect_error(
    normalize("unsupported", "beta", "unknown"),
    "arg.*one of"
  )
})

test_that("unknown-theta method is rejected when it is not applicable", {
  normalize <- AccSamplingDesign:::.normalize_beta_theta_method

  expect_null(normalize(NULL, "normal", "unknown", method_missing = TRUE))
  expect_null(normalize(NULL, "beta", "known", method_missing = TRUE))
  expect_error(
    normalize("delta_mle", "normal", "unknown"),
    "only applicable"
  )
  expect_error(
    normalize("delta_mle", "beta", "known"),
    "only applicable"
  )
})

test_that("Beta raw moments match closed-form reference values", {
  moments <- AccSamplingDesign:::.beta_raw_moments(0.25, 20, 4)

  # For Beta(5, 15), E[Y^r] = (5)_r / (20)_r.
  expected <- c(
    5 / 20,
    (5 * 6) / (20 * 21),
    (5 * 6 * 7) / (20 * 21 * 22),
    (5 * 6 * 7 * 8) / (20 * 21 * 22 * 23)
  )
  expect_equal(moments, expected, tolerance = 1e-14)
})

test_that("analytical MoM covariance is finite and symmetric", {
  covariance <- AccSamplingDesign:::.beta_mom_covariance(0.25, 20)

  # Frozen from direct raw-moment propagation for Beta(5, 15).
  expected <- matrix(
    c(0.00892857142857143, -0.454545454545456,
      -0.454545454545456, 834.466403162061),
    nrow = 2,
    byrow = TRUE
  )
  expect_equal(covariance, expected, tolerance = 1e-10)
  expect_equal(covariance, t(covariance), tolerance = 1e-14)
  expect_true(all(eigen(covariance, symmetric = TRUE)$values >= -1e-10))
})

test_that("Beta moment helpers reject invalid parameters", {
  raw_moments <- AccSamplingDesign:::.beta_raw_moments
  mom_covariance <- AccSamplingDesign:::.beta_mom_covariance

  expect_error(raw_moments(0, 20), "mu")
  expect_error(raw_moments(0.5, -1), "theta")
  expect_error(raw_moments(0.5, 20, 0), "max_order")
  expect_error(mom_covariance(1, 20), "mu")
})

test_that("analytical MLE covariance inverts Beta Fisher information", {
  information <- AccSamplingDesign:::.beta_mle_information(0.25, 20)
  covariance <- AccSamplingDesign:::.beta_mle_covariance(0.25, 20)

  expect_equal(information, t(information), tolerance = 1e-14)
  expect_equal(covariance, t(covariance), tolerance = 1e-14)
  expect_equal(information %*% covariance, diag(2), tolerance = 1e-10)
  expect_true(all(eigen(covariance, symmetric = TRUE)$values > 0))
})

test_that("analytical MLE covariance matches a frozen reference", {
  covariance <- AccSamplingDesign:::.beta_mle_covariance(0.25, 20)

  # Inverse of the expected (mu, theta) Fisher information for Beta(5, 15).
  expected <- matrix(
    c(0.00891453953147548, -0.482730372568412,
      -0.482730372568412, 772.623438103779),
    nrow = 2,
    byrow = TRUE
  )
  expect_equal(covariance, expected, tolerance = 1e-6)
})

test_that("Beta MLE covariance rejects invalid parameters", {
  mle_information <- AccSamplingDesign:::.beta_mle_information
  mle_covariance <- AccSamplingDesign:::.beta_mle_covariance

  expect_error(mle_information(NA_real_, 20), "mu")
  expect_error(mle_information(0.5, 0), "theta")
  expect_error(mle_covariance(1, 20), "mu")
})

test_that("Delta decision gradients agree with finite differences", {
  statistic <- AccSamplingDesign:::.beta_delta_statistic
  gradient <- AccSamplingDesign:::.beta_delta_gradient
  mu <- 0.25
  theta <- 20
  k <- 1.4
  step <- 1e-6

  for (limit_type in c("upper", "lower")) {
    analytical <- gradient(mu, theta, k, limit_type)
    numerical_mu <- (statistic(mu + step, theta, k, limit_type) -
      statistic(mu - step, theta, k, limit_type)) / (2 * step)
    numerical_theta <- (statistic(mu, theta + step, k, limit_type) -
      statistic(mu, theta - step, k, limit_type)) / (2 * step)

    expect_equal(unname(analytical[["mu"]]), numerical_mu, tolerance = 1e-7)
    expect_equal(
      unname(analytical[["theta"]]), numerical_theta, tolerance = 1e-7
    )
  }
})

test_that("Delta acceptance probabilities match frozen references", {
  delta_pa <- AccSamplingDesign:::.beta_delta_acceptance_probability

  expect_equal(
    delta_pa(0.03, 300, 45, 2.2, 0.05, "upper", "delta_mle"),
    0.312756442044726,
    tolerance = 1e-6
  )
  expect_equal(
    delta_pa(0.08, 100, 30, 1.5, 0.05, "lower", "delta_mom"),
    0.0404806618576654,
    tolerance = 1e-6
  )
})

test_that("Delta acceptance probability validates numerical inputs", {
  delta_pa <- AccSamplingDesign:::.beta_delta_acceptance_probability

  expect_error(delta_pa(0.3, 20, 0, 1, 0.5, "upper", "delta_mle"), "n")
  expect_error(delta_pa(0.3, 20, 10, -1, 0.5, "upper", "delta_mle"), "k")
  expect_error(delta_pa(0.3, 20, 10, 1, 1, "upper", "delta_mle"), "limit")
  expect_error(delta_pa(0.3, 20, 10, 1, 0.5, "upper", "bad"), "arg")
})

## -----------------------------------------------------------------------------
## optAttrPlan.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: Design attribute acceptance sampling plans (binomial)
##
## Changelogs:
## -----------------------------------------------------------------------------

# find_plan_binomial <- function(PRQ, CRQ, alpha, beta, measurement_error = 0) {
#   # Input validation
#   if(PRQ >= CRQ) stop("PRQ must be < CRQ")
#   if(alpha <= 0 || alpha >= 1) stop("alpha must be in (0,1)")
#   if(beta <= 0 || beta >= 1) stop("beta must be in (0,1)")
#   
#   n_vec <- 1:1e5 #define Maximum vector n
#   
#   # Vectorized qbinom to find c for all n
#   c_vec <- qbinom(1 - alpha, n_vec, PRQ, lower.tail = TRUE)
#   
#   for (i in seq_along(n_vec)) {
#     n <- n_vec[i]
#     c <- c_vec[i]
#     pa_c <- pbinom(c, n, CRQ)
#     pa_p <- pbinom(c, n, PRQ)
#     if (pa_c <= beta) {
#       return(structure(
#         list(
#           n = n,
#           c = c,
#           PR = 1 - pa_p,
#           CR = pa_c,
#           PRQ = PRQ,
#           CRQ = CRQ,
#           sample_size = ceiling(n),
#           distribution = "binomial",
#           measurement_error = measurement_error
#         ),
#         class = "AttrPlan"
#       ))
#     }
#   }
#   stop("No solution found!")
# }

find_plan_binomial <- function(PRQ, CRQ, alpha, beta, measurement_error = 0) {
  
  if (PRQ >= CRQ) stop("PRQ must be < CRQ")
  if (alpha <= 0 || alpha >= 1) stop("alpha must be in (0,1)")
  if (beta  <= 0 || beta  >= 1) stop("beta must be in (0,1)")
  if (!is.numeric(measurement_error) || length(measurement_error) != 1)
    stop("measurement_error must be a numeric scalar")
  
  n_vec <- 1:1e5
  
  c_vec <- qbinom(1 - alpha, n_vec, PRQ, lower.tail = TRUE)
  
  # Ensure valid c values
  c_vec <- pmax(c_vec, 0)
  
  pa_c_vec <- pbinom(c_vec, n_vec, CRQ)
  idx <- which(pa_c_vec <= beta)[1]
  
  if (is.na(idx)) stop("No solution found!")
  
  n <- n_vec[idx]
  c <- c_vec[idx]
  pa_p <- pbinom(c, n, PRQ)
  
  structure(
    list(
      n = n,
      c = c,
      PR = 1 - pa_p,
      CR = pa_c_vec[idx],
      PRQ = PRQ,
      CRQ = CRQ,
      sample_size = n,
      distribution = "binomial",
      measurement_error = measurement_error
    ),
    class = "AttrPlan"
  )
}


# find_plan_poisson <- function(PRQ, CRQ, alpha, beta, measurement_error = 0) {
#   # Input validation
#   if (PRQ >= CRQ) stop("PRQ must be < CRQ")
#   if (alpha <= 0 || alpha >= 1) stop("alpha must be in (0,1)")
#   if (beta <= 0 || beta >= 1) stop("beta must be in (0,1)")
#   
#   max_n <- 1e5
#   for (n in 1:max_n) {
#     lambda_prq <- n * PRQ
#     lambda_crq <- n * CRQ
#     
#     # Find smallest c satisfying producer’s risk
#     c <- qpois(1 - alpha, lambda_prq, lower.tail = TRUE)
#     
#     pa_p <- ppois(c, lambda_prq)  # producer's risk at PRQ
#     pa_c <- ppois(c, lambda_crq)  # consumer's risk at CRQ
#     
#     if (pa_p >= 1 - alpha && pa_c <= beta) {
#       return(structure(
#         list(
#           n = n,
#           c = c,
#           PR = 1 - pa_p,
#           CR = pa_c,
#           PRQ = PRQ,
#           CRQ = CRQ,
#           sample_size = ceiling(n),
#           distribution = "poisson",
#           measurement_error = measurement_error
#         ),
#         class = "AttrPlan"
#       ))
#     }
#   }
#   
#   stop("No solution found within max_n = ", max_n)
# }

find_plan_poisson <- function(PRQ, CRQ, alpha, beta, measurement_error = 0) {
  
  # --- Input validation ---
  if (PRQ >= CRQ) stop("PRQ must be < CRQ")
  if (alpha <= 0 || alpha >= 1) stop("alpha must be in (0,1)")
  if (beta  <= 0 || beta  >= 1) stop("beta must be in (0,1)")
  if (!is.numeric(measurement_error) || length(measurement_error) != 1)
    stop("measurement_error must be a numeric scalar")
  
  # --- Vectorized n ---
  n_vec <- 1:1e5
  
  # λ for PRQ and CRQ
  lambda_prq_vec <- n_vec * PRQ
  lambda_crq_vec <- n_vec * CRQ
  
  # --- Compute c(n) under PRQ ---
  c_vec <- qpois(1 - alpha, lambda_prq_vec)
  
  # --- Compute Pa under PRQ and CRQ ---
  pa_p_vec <- ppois(c_vec, lambda_prq_vec)  # producer's acceptance probability
  pa_c_vec <- ppois(c_vec, lambda_crq_vec)  # consumer's acceptance probability
  
  # --- Condition: PR satisfied & CR satisfied ---
  # PR: pa_p >= 1 - alpha
  # CR: pa_c <= beta
  idx <- which(pa_p_vec >= 1 - alpha & pa_c_vec <= beta)[1]
  
  if (is.na(idx)) stop("No solution found!")
  
  # Extract optimal values
  n <- n_vec[idx]
  c <- c_vec[idx]
  
  structure(
    list(
      n = n,
      c = c,
      PR = 1 - pa_p_vec[idx],
      CR = pa_c_vec[idx],
      PRQ = PRQ,
      CRQ = CRQ,
      sample_size = n,
      distribution = "poisson",
      measurement_error = measurement_error
    ),
    class = "AttrPlan"
  )
}


#' Attribute Acceptance Sampling Plan
#' @export
optAttrPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10, 
                        distribution = c("binomial", "poisson")) {
  distribution <- match.arg(distribution)
  # Input validation
  if(PRQ >= CRQ) stop("PRQ must be < CRQ")
  if(alpha <= 0 || alpha >= 1) stop("alpha must be in (0,1)")
  if(beta <= 0 || beta >= 1) stop("beta must be in (0,1)")
  
  # Init an optimal plan
  attr_plan = NULL
  
  # ------------Binomial ----------
  if (distribution == "binomial") {
    attr_plan <- find_plan_binomial(PRQ = PRQ, CRQ = CRQ, 
                                    alpha = alpha, beta = beta)
  }
  # ------------Poisson ------------
  if (distribution == "poisson") {
    attr_plan <- find_plan_poisson(PRQ = PRQ, CRQ = CRQ, 
                                   alpha = alpha, beta = beta)
  }
  return(attr_plan)
}


#' @export
plot.AttrPlan <- function(x, pd = NULL, ...) {
  if (is.null(pd)) {
    pd <- seq(1e-10, min(x$CRQ * 2, 1), length.out = 100)
  }
  pa <- sapply(pd, function(p) accProb(x, p))
  
  plot(pd, pa, type = "l", col = "blue", lwd = 2,
       main = paste0("Attributes Sampling OC Curve", " | n=", 
                     x$sample_size, ", c=", x$c, " | ", x$distribution),
       xlab = "Proportion Nonconforming", ylab = "P(accept)", ...)
  abline(v = c(x$PRQ, x$CRQ), lty = 2, col = "gray")
  abline(h = c(1 - x$PR, x$CR), lty = 2, col = "gray")
  grid()
}

## -----------------------------------------------------------------------------
## test-plots.R
##
## Author: Ha Truong
## Created: 12 Nov 2025
## Purpose: Test all plot functions
## -----------------------------------------------------------------------------



# Tests for plot.AttrPlan()
# -----------------------------------------------------------------------------

test_that("plot.AttrPlan runs without error", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  
  expect_silent({
    plot(plan)
  })
})

test_that("plot.AttrPlan runs with custom pd", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  pd <- seq(0, 0.1, length.out = 20)
  
  expect_silent({
    plot(plan, pd = pd)
  })
})


test_that("plot.AttrPlan produces a plot object", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  
  expect_silent({
    plot(plan)
  })
  
  p <- recordPlot()
  expect_true(inherits(p, "recordedplot"))
})


test_that("accProb agrees with binomial pbinom", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  p <- 0.03
  
  expect_equal(
    accProb(plan, p),
    pbinom(plan$c, plan$n, p),
    tolerance = 1e-6
  )
})


# Tests for plot.VarPlan()
# --------------------------------------------------------------------

test_that("plot.VarPlan (by='pd') runs without error", {
  plan <- optVarPlan(PRQ = 0.005, CRQ = 0.03,
                     alpha = 0.05, beta = 0.10,
                     distribution = "normal", sigma_type = "known")
  
  expect_silent({
    plot(plan, by = "pd")
  })
})

test_that("plot.VarPlan (by='pd') works with custom pd", {
  plan <- optVarPlan(PRQ = 0.005, CRQ = 0.03,
                     alpha = 0.05, beta = 0.10,
                     distribution = "normal", sigma_type = "known")
  
  pd <- seq(0, 0.1, length.out = 20)
  
  expect_silent({
    plot(plan, pd = pd, by = "pd")
  })
})

test_that("plot.VarPlan produces a recordedplot (by='pd')", {
  plan <- optVarPlan(PRQ = 0.005, CRQ = 0.03,
                     alpha = 0.05, beta = 0.10,
                     distribution = "normal", sigma_type = "known")
  
  plot(plan, by = "pd")
  p <- recordPlot()
  
  expect_true(inherits(p, "recordedplot"))
})

test_that("plot.VarPlan errors when sigma_type missing", {
  plan <- list(
    PRQ = 0.01, CRQ = 0.05,
    distribution = "normal",
    sigma = 1, USL = 10, LSL = 0
  )
  class(plan) <- "VarPlan"
  
  expect_error(plot(plan), "sigma_type")
})

test_that("plot.VarPlan errors when sigma missing", {
  plan <- list(PRQ = 0.01, CRQ = 0.05, distribution = "normal")
  class(plan) <- "VarPlan"
  
  expect_error(plot(plan), "sigma")
})


test_that("plot.VarPlan errors for invalid by argument", {
  plan <- optVarPlan(PRQ = 0.005, CRQ = 0.03,
                     alpha = 0.05, beta = 0.10,
                     distribution = "normal", sigma_type = "known",
                     USL = 0.05)
  
  expect_error(plot(plan, by="xyz"), "should be one of")
})

test_that("plot.VarPlan supports every unknown-theta Beta method", {
  pd <- seq(0.005, 0.08, length.out = 12)
  for (method in c("delta_mle", "delta_mom", "gk_adjustment")) {
    plan <- manualPlan(
      distribution = "beta", n = 45, k = 2.2,
      theta = 300, theta_type = "unknown", method = method,
      USL = 0.05
    )
    plan$PRQ <- 0.01
    plan$CRQ <- 0.05
    plan$PR <- 0.05
    plan$CR <- 0.10

    expect_silent(plot(plan, pd = pd, by = "pd"))
    expect_silent(plot(plan, pd = pd, by = "mean"))
  }
})

## -----------------------------------------------------------------------------
## test-optAttrPlan.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: Test attribute plan calculations
##
## Changelogs:
## -----------------------------------------------------------------------------

test_that("Attribute plan meets risk requirements", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  expect_lte(pbinom(plan$c, plan$n, 0.05), 0.10)  # Verify beta
  expect_gte(pbinom(plan$c, plan$n, 0.01), 0.95)  # Verify 1-alpha
})

test_that("optAttrPlan returns valid binomial plan", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  
  expect_true(is.list(plan))
  expect_true(all(c("n", "c") %in% names(plan)))
  expect_gt(plan$n, 0)
  expect_gte(plan$c, 0)
})

test_that("optAttrPlan returns valid poisson plan", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "poisson")
  
  expect_true(is.list(plan))
  expect_true(all(c("n", "c") %in% names(plan)))
  expect_gt(plan$n, 0)
  expect_gte(plan$c, 0)
})

test_that("optAttrPlan rejects invalid alpha or beta", {
  expect_error(optAttrPlan(PRQ = 0.01, CRQ = 0.05, alpha = -0.1))
  expect_error(optAttrPlan(PRQ = 0.01, CRQ = 0.05, beta = 1.1))
})

# test_that("optAttrPlan dispatches binomial to find_plan_binomial", {
#   mock <- mockery::mock(list(n = 10, c = 1))
#   mockery::stub(optAttrPlan, "find_plan_binomial", mock)
#   
#   optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
#   mockery::expect_called(mock, 1)
# })

test_that("optAttrPlan plan meets risk constraints (binomial)", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
  
  # Consumer's risk: P(accept at CRQ) ≤ beta
  pa_crq <- pbinom(plan$c, plan$n, 0.05)
  expect_lte(pa_crq, 0.10)
  
  # Producer's risk: P(accept at PRQ) ≥ 1 - alpha
  pa_prq <- pbinom(plan$c, plan$n, 0.01)
  expect_gte(pa_prq, 0.95)
})

## -----------------------------------------------------------------------------
## optVarPlan.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: Design variable acceptance sampling plans (normal/beta)
##
## Changelogs:
## 19 Apr, 2025: added S3 plot method
## 20 Apr, 2025: remove spec_limit and limit_type. Replace by USL/LSL
## -----------------------------------------------------------------------------


# Define optimization function to minimize difference between Pa and target 
# values for alpha and beta (t-distribution)
t_optimize_sampling_plan <- function(alpha, beta, p_alpha, p_beta, n_init, k_init) 
{
  # Define function to calculate Pa based on the non-central t-distribution
  Pa_tdist <- function(n, k, p) {
    # enforce n >= 2 to have valid df = n - 1
    n <- max(2, n)
    # Calculate the acceptance probability using non-central t-distribution
    non_central_t <- pt(k * sqrt(n), df = n - 1, ncp = -qnorm(p) * sqrt(n))
    return(1 - non_central_t)
  }
  
  # Objective function to minimize
  objective_function <- function(params) {
    n_s <- params[1]
    k_s <- params[2]
    
    # Calculate Pa for both p_alpha and p_beta
    Pa_alpha <- Pa_tdist(n_s, k_s, p_alpha)
    Pa_beta <- Pa_tdist(n_s, k_s, p_beta)
    
    # Minimize the differences from target acceptance probabilities
    return(abs(Pa_alpha - (1 - alpha)) + abs(Pa_beta - beta))
  }
  
  # Initial guess for n_s and k_s
  init_params <- c(n_init, k_init)  # Example initial guess
  
  # Solve the optimization problem
  result <- optim(init_params, objective_function)
  
  # Handle cases where the optimization fails to find an optimal solution
  if (result$convergence != 0) {
    return(list(
      success = FALSE,
      code    = result$convergence,
      message = result$message
    ))
  }
  
  # Optimal plan found!
  return(list(success = TRUE, n = result$par[1], k = result$par[2]))
  #return(result$par)  # Return optimized n_s and k_s
}

#' Variable Acceptance Sampling Plan
#' @export
optVarPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10,
                       USL = NULL, LSL = NULL,
                       distribution = c("normal", "beta"),
                       sigma_type = c("known", "unknown"),
                       theta_type = c("known", "unknown"),
                       sigma = NULL, theta = NULL,
                       method = c("delta_mle", "delta_mom", "gk_adjustment")) {
  method_missing <- missing(method)
  
  # Match arguments to ensure valid input
  distribution <- match.arg(distribution)
  sigma_type <- match.arg(sigma_type)
  theta_type <- match.arg(theta_type)
  method <- .normalize_beta_theta_method(
    method = if (method_missing) NULL else method,
    distribution = distribution,
    theta_type = theta_type,
    method_missing = method_missing
  )
  
  # Set default if not provided
  if (is.null(sigma_type)) sigma_type <- "known"
  if (is.null(theta_type)) theta_type <- "known"
  
  if (!is.null(USL) && !is.null(LSL)) {
    stop("Double specification limits (both USL and LSL) are not supported. 
         Please specify only one limit: either USL or LSL.")
  }
  # This upper limit case
  if (!is.null(USL)) {
    limit_type <- "upper"
    spec_limit <- USL
  } else if (!is.null(LSL)) { # This lower limit case
    limit_type <- "lower"
    spec_limit <- LSL
  } else {
    limit_type <- "upper" # set this as default
  }
  
  # Ensure PRQ, CRQ, alpha, and beta are within valid ranges based on distribution
  check_quality <- function(q, distribution) {
    if (distribution == "normal" && (q < 0 || q > 1)) {
      return(FALSE)
    }
    if (distribution == "beta" && (q <= 0 || q >= 1)) {
      return(FALSE)
    }
    return(TRUE)
  }
  
  if (!check_quality(PRQ, distribution) || !check_quality(CRQ, distribution)) {
    stop("PRQ and CRQ are out of bounds for the selected distribution.")
  }
  
  if (alpha <= 0 || alpha >= 1 || beta <= 0 || beta >= 1) {
    stop("alpha and beta must be between 0 and 1 (exclusive).")
  }
  
  # Ensure PRQ and CRQ are provided
  if (missing(PRQ) || missing(CRQ)) {
    stop("Producer and Consumer Risk Points must be provided.")
  }
  
  # Additional checks for beta distribution
  if (distribution == "beta") {
    if(is.null(USL) && is.null(LSL)){
      stop("For the beta distribution, a specification limit must be provided.")
    }
    if (is.null(theta)) {
      stop("For the beta distribution, theta must be provided.")
    }
  }
  
  # Ensure Consumer Risk Quality (CRQ) > Producer Risk Quality (PRQ)
  if (CRQ <= PRQ) {
    stop("Consumer Risk Quality (CRQ) must be greater than Producer Risk Quality (PRQ).")
  }
  
  # Check measurement error is non-negative
  # if (measurement_error < 0) {
  #   stop("measurement_error must be non-negative.")
  # }
  
  if (distribution == "normal") {
    # Define quantile functions for standard normal
    u_alpha <- qnorm(1 - alpha)
    u_beta <- qnorm(1 - beta)
    u_p1 <- qnorm(1 - PRQ)
    u_p2 <- qnorm(1 - CRQ)
    
    # Compute sample size n
    n <- ((u_alpha + u_beta) / (u_p1 - u_p2))^2
    # Compute acceptability constant k
    k <- (u_p1 * u_beta + u_p2 * u_alpha) / (u_alpha + u_beta)
    
    if(sigma_type == "unknown") {
      # find n_s by approximate from n
      # n <- ceiling(n) * (1 + k^2/2)
      
      # init value of n, k for optimization process
      n_init <- max(2, n)
      k_init <- k
      
      # # Get optimal n_s and k_s set n,k as init value for optimize function
      # optimal_params <- t_optimize_sampling_plan(alpha, beta, PRQ, CRQ, 
      #                                            n_init, k_init) 
      # # final plan (n,k) after optimized
      # n = optimal_params[1]
      # k = optimal_params[2]
      
      # Get optimal n_s and k_s by optimizing
      opt <- t_optimize_sampling_plan(alpha, beta, PRQ, CRQ, n_init, k_init)
      
      # Check if optimization was successful
      if (!opt$success) {
        stop(
          paste("Optimization failed:",
                opt$message,
                "Please check inputs: alpha/beta, PRQ/CRQ or try different initial values.")
        )
      }
      
      # If success, set final plan
      n <- opt$n
      k <- opt$k
    }
    
    sample_size <- n
    
    objPlan <- structure(list(n = n, k = k, sigma_type = sigma_type,
                              distribution = distribution), 
                         class = "VarPlan")
    
    r_alpha <- 1 - accProb(objPlan, PRQ)
    r_beta <- accProb(objPlan, CRQ)
    #m <- NA  # Not applicable for normal
  } else {
    
    # Constraint function: Ensure PR <= alpha and CR <= beta
    constraint_function <- function(params) {
      n <- params[1]
      k <- params[2]
      
      evaluation_theta_type <- if (
        theta_type == "unknown" && method != "gk_adjustment"
      ) "unknown" else "known"
      planObj <- structure(list(n = n, k = k, USL = USL, LSL = LSL,
                                theta = theta,
                                theta_type = evaluation_theta_type,
                                method = method,
                                distribution = distribution), 
                           class = "VarPlan")
      
      PR <- 1 - accProb(planObj, PRQ)
      CR <- accProb(planObj, CRQ)
      
      # Apply large penalty if constraints are violated
      penalty <- (pmax(PR - alpha, 0))^2 + (pmax(CR - beta, 0))^2 # smoother penalty 
      
      # Check if constraints are satisfied
      if (PR <= alpha && CR <= beta) {
        return(n)  # Return actual objective if feasible
      } else {
        # Return very large number if constraints violated
        return(n + 1e4 * penalty * 7)   # (objective + penalty combined)
      }
    }
    
    # Get initial estimates using normal distribution
    normal_plan <- optVarPlan(PRQ = PRQ, CRQ = CRQ, distribution = "normal")
    
    # Define search ranges
    n_range <- c(floor(0.5 * normal_plan$n), ceiling(2 * normal_plan$n))
    k_range <- c(max(0.5 * normal_plan$k, 0.1), 2 * normal_plan$k)
    
    # Initial guess 
    initial_guess <- c(normal_plan$n, normal_plan$k)
    
    #Run optimization using optim (L-BFGS-B supports bounds)
    result <- optim(
      par = initial_guess,
      fn = constraint_function,
      method = "L-BFGS-B",
      lower = c(n_range[1], k_range[1]),
      upper = c(n_range[2], k_range[2])
    )
    
    if (result$convergence == 0) {
      # full success
      n_opt <- result$par[1]
      k_opt <- result$par[2]
      
    } else if (result$convergence == 51) {
      # warning but solution likely usable
      warning("Optimization returned a warning (code 51): ", result$message)
      n_opt <- result$par[1]
      k_opt <- result$par[2]
      
    } else if (result$convergence == 52) {
      # error — check if par is still valid
      if (all(is.finite(result$par))) {
        warning("Optimization returned a warning (code 52): ", result$message)
        n_opt <- result$par[1]
        k_opt <- result$par[2]
      } else {
        stop("Optimization failed with convergence code 52: ", result$message)
      }
      
    } else {
      stop("Optimization failed with convergence code:", result$convergence)
    }
    
    # Extract optimal values
    # n <- result$par[1]
    # k <- result$par[2]
    n <- n_opt
    k <- k_opt

    if (theta_type == "unknown" && method != "gk_adjustment") {
      # For Delta methods both risk equations are smooth. Refine the generic
      # constrained search by solving for the two binding risk points in
      # log(n), k coordinates, which keeps n positive and improves numerical
      # accuracy for very large precision parameters.
      delta_risk_error <- function(transformed) {
        candidate <- structure(
          list(
            n = exp(transformed[[1L]]), k = exp(transformed[[2L]]),
            USL = USL, LSL = LSL, theta = theta,
            theta_type = "unknown", method = method,
            distribution = "beta"
          ),
          class = "VarPlan"
        )
        producer_risk <- 1 - accProb(candidate, PRQ)
        consumer_risk <- accProb(candidate, CRQ)
        ((producer_risk - alpha) / alpha)^2 +
          ((consumer_risk - beta) / beta)^2
      }
      refined <- optim(
        par = log(c(max(n, 2), k)),
        fn = delta_risk_error,
        method = "Nelder-Mead",
        control = list(maxit = 1000, reltol = 1e-12)
      )
      if (refined$convergence != 0 || any(!is.finite(refined$par))) {
        stop("Delta-method risk refinement failed: ", refined$message)
      }
      n <- exp(refined$par[[1L]])
      k <- exp(refined$par[[2L]])
    }
    
    if(theta_type == "unknown" && method == "gk_adjustment") {
      # Preserve the package-style adjustment from version 0.0.8. Delta
      # methods instead incorporate estimation uncertainty directly in Pa.
      R_ratio <- 1 + 0.85 * k^2
      n <- ceiling(n) * R_ratio
    }
    
    objPlan <- structure(list(n = n, k = k, USL = USL, LSL = LSL,
                              theta = theta, theta_type = theta_type,
                              method = method,
                              distribution = distribution), 
                         class = "VarPlan")
    
    r_alpha <- 1 - accProb(objPlan, PRQ)
    r_beta <- accProb(objPlan, CRQ)
    sample_size <- n
    #n <- m  # Not applicable for beta
  }
  
  return(structure(
    list(
      distribution = distribution,
      sigma = sigma, theta = theta,
      sigma_type = sigma_type, theta_type = theta_type, method = method,
      PRQ = PRQ, CRQ = CRQ, PR = r_alpha, CR = r_beta,
      USL = USL, LSL = LSL,
      #spec_limit = spec_limit, limtype = limit_type,
      sample_size = ceiling(sample_size), # round up the sample size for practical;
      n = n, # we keep the number before round up
      #m = m, # we keep the number before round up
      k = k
    ), 
    class = "VarPlan"
  ))
}

#' @export
plot.VarPlan <- function(x, pd = NULL, by = c("pd", "mean"), ...) {
  by <- match.arg(by)
  
  # Default pd if not supplied
  if (is.null(pd)) {
    pd <- seq(max(x$PRQ * 0.5, 1e-10), min(x$CRQ * 1.5, 1), length.out = 100)
  }
  
  if (is.null(x$sigma_type)) {
    stop("sigma_type must be provided to plot a VarPlan")
  }
  
  if (by == "pd") {
    pa <- sapply(pd, function(p) accProb(x, p))
    plot(pd, pa, type = "l", col = "red", lwd = 2,
         main = paste0("Variables Sampling OC Curve by Pd",
                       " | n=", x$sample_size, ", k=", round(x$k,3), " | ", x$distribution),
         xlab = "Proportion Nonconforming", ylab = "P(accept)", ...)
    abline(v = c(x$PRQ, x$CRQ), lty = 2, col = "gray")
    abline(h = c(1 - x$PR, x$CR), lty = 2, col = "gray")
    grid()
    
  } else if (by == "mean") {
    # Check that spec limits and limit_type exist
    if (is.null(x$USL) && is.null(x$LSL)) {
      stop("Mean-level plot requires 'USL' or 'LSL' in the plan.")
    }
    # estimate mu from PRQ/CRQ
    mu_PRQ <- muEst(x$PRQ, USL = x$USL, LSL = x$LSL, sigma = x$sigma, 
                    theta = x$theta, dist = x$distribution)
    mu_CRQ <- muEst(x$CRQ, USL = x$USL, LSL = x$LSL, sigma = x$sigma, 
                    theta = x$theta, dist = x$distribution)      
    # Estimate mean levels based on pd
    mu_vals <- sapply(pd, function(p) muEst(
      p, USL = x$USL, LSL = x$LSL,
      sigma = x$sigma, theta = x$theta, 
      dist = x$distribution
    ))
    
    if (any(is.na(mu_vals))) {
      stop("Some mean estimates could not be computed. Check the pd or spec limits.")
    }
    
    pa <- sapply(pd, function(p) accProb(x, p))
    plot(mu_vals, pa, type = "l", col = "blue", lwd = 2,
         main = paste0("Variables Sampling OC Curve by Mean",
                       " | n=", x$sample_size, ", k=", round(x$k,3), " | ", x$distribution),
         xlab = "Process Mean", ylab = "P(accept)", ...)
    abline(v = c(mu_PRQ, mu_CRQ), lty = 2, col = "gray")
    abline(h = c(1 - x$PR, x$CR), lty = 2, col = "gray")
    grid()
  }
}

## -----------------------------------------------------------------------------
## test-optVarPlan.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: Test variable plan calculations
##
## Changelogs:
## -----------------------------------------------------------------------------

test_that("Normal plan and known sigma - creates valid parameters", {
  plan <- optVarPlan(PRQ = 0.005, CRQ = 0.03, alpha = 0.05, beta = 0.10, 
                     distribution = "normal", sigma_type = "known")
  expect_gt(plan$k, 0)
  expect_gt(plan$n, 0)
})

test_that("Normal plan and unknown sigma - creates valid parameters", {
  plan <- optVarPlan(PRQ = 0.005, CRQ = 0.03, alpha = 0.05, beta = 0.10, 
                     distribution = "normal", sigma_type = "unknown")
  expect_gt(plan$k, 0)
  expect_gte(plan$n, 2)
})

test_that("unknown-theta Beta plans support all methods", {
  plans <- lapply(
    c("delta_mle", "delta_mom", "gk_adjustment"),
    function(method) {
      optVarPlan(
        PRQ = 0.025, CRQ = 0.10, alpha = 0.05, beta = 0.10,
        distribution = "beta", theta_type = "unknown", theta = 6.6e8,
        LSL = 5.65e-6, method = method
      )
    }
  )

  expect_identical(vapply(plans, `[[`, character(1), "method"),
                   c("delta_mle", "delta_mom", "gk_adjustment"))
  for (plan in plans) {
    expect_true(is.finite(plan$n) && plan$n > 0)
    expect_true(is.finite(plan$k) && plan$k > 0)
    expect_true(is.finite(plan$PR) && is.finite(plan$CR))
    expect_lte(plan$PR, 0.055)
    expect_lte(plan$CR, 0.105)
  }
})

test_that("Delta-MLE is the default unknown-theta Beta method", {
  plan <- optVarPlan(
    PRQ = 0.025, CRQ = 0.10,
    distribution = "beta", theta_type = "unknown", theta = 6.6e8,
    LSL = 5.65e-6
  )
  expect_identical(plan$method, "delta_mle")
})

test_that("Delta plan optimization supports an upper specification limit", {
  plan <- optVarPlan(
    PRQ = 0.01, CRQ = 0.05, alpha = 0.05, beta = 0.10,
    distribution = "beta", theta_type = "unknown", theta = 300,
    USL = 0.05, method = "delta_mle"
  )

  expect_lte(plan$PR, 0.0505)
  expect_lte(plan$CR, 0.1005)
})

test_that("method is rejected outside unknown-theta Beta plans", {
  expect_error(
    optVarPlan(0.01, 0.05, distribution = "normal", method = "delta_mle"),
    "only applicable"
  )
  expect_error(
    optVarPlan(0.01, 0.05, distribution = "beta", theta = 100, USL = 0.1,
               method = "delta_mle"),
    "only applicable"
  )
})

## -----------------------------------------------------------------------------
## muEst.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: estimate mu corresponding to a given proportion nonconforming (p)
##
## Changelogs:
## -----------------------------------------------------------------------------

#' Estimate mu base on quality level pre-defined (like PRQ/CRQ)
#' @export
muEst <- function(p, USL = NULL, LSL = NULL, 
                  sigma = NULL, theta = NULL, 
                  dist = c("normal", "beta")) {
  dist <- match.arg(dist)
  ##limtype <- match.arg(limtype)
  if (is.null(USL) && is.null(LSL)){
    stop("A specification limit (USL or LSL) is required for mu estimation!")
  }
  # This upper limit case
  if (!is.null(USL)) {
    limtype <- "upper"
    limit <- USL
  } else if (!is.null(LSL)) { # This lower limit case
    limtype <- "lower"
    limit <- LSL
  } else {
    limtype <- "upper" # set this as default
  }
  
  if (dist == "normal") {
    if (is.null(sigma)) stop("MuEST failed: sigma must be provided for normal distribution")
    if (limtype == "lower") {
      mu <- limit - sigma * qnorm(p)
    } else {
      mu <- limit - sigma * qnorm(1 - p)
    }
    return(mu)
  }
  
  if (dist == "beta") {
    if (is.null(theta)) stop("MuEST failed: theta must be provided for beta distribution")
    
    # Handle edge case where limit = 0
    if (limit == 0) {
      if (limtype == "lower") {
        return(NA)  # No solution exists
      } else {
        return(1)  # If limit = 0 and "upper", mu should be 1
      }
    }
    
    # Define function to root-find based on limit type
    if (limtype == "lower") {
      f <- function(x) pbeta(limit, theta * x, theta * (1 - x)) - p
    } else {
      f <- function(x) 1 - pbeta(limit, theta * x, theta * (1 - x)) - p
    }
    
    # Find root numerically
    uniroot_result <- uniroot(f, interval = c(0, 1), tol = 1e-9)
    return(uniroot_result$root)
  }
}

## -----------------------------------------------------------------------------
## manualPlan.R --- 
##
## Author: Ha Truong
##
## Created: 07 July 2025
##
## Purposes: Generate munual varPlan/attrPlan objs from parameters
##
## Changelogs:
## -----------------------------------------------------------------------------


#' Create an OCdata object
#' @export
manualPlan <- function(distribution = c("binomial", "poisson", "normal", "beta"),
                       n = NULL, c = NULL, k = NULL,
                       USL = NULL, LSL = NULL, sigma = NULL, theta = NULL, 
#                       PRQ = NULL, CRQ = NULL, alpha = NULL, beta = NULL,
                       sigma_type = c("known", "unknown"),
                       theta_type = c("known", "unknown"),
                       method = c("delta_mle", "delta_mom", "gk_adjustment")) {
  method_missing <- missing(method)
  sigma_type <- match.arg(sigma_type)
  theta_type <- match.arg(theta_type)
  distribution <- match.arg(distribution)
  method <- .normalize_beta_theta_method(
    method = if (method_missing) NULL else method,
    distribution = distribution,
    theta_type = theta_type,
    method_missing = method_missing
  )
  
  # Use for placeholder only
  PRQ = NULL 
  CRQ = NULL 
  alpha = NULL
  beta = NULL
  
  if (!(distribution %in% c("binomial", "poisson", "normal", "beta"))) {
    stop("Unsupported distribution.")
  }
  # Assign NULL first
  plan <- NULL
  
  if (distribution %in% c("binomial", "poisson")) {
    if (is.null(n) || is.null(c)) stop("n and c must be provided.")
    plan <- structure(list(n = n, c = c, sample_size = ceiling(n),
                           PRQ = PRQ, CRQ = CRQ, PR = alpha, CR = beta,
                           USL = USL, LSL = LSL,
                           distribution = distribution),
                      class = "AttrPlan")
  }
  
  if (distribution %in% c("normal", "beta")) {
    if (is.null(n) || is.null(k)) stop("n and k must be provided.")
    if (distribution == "beta" && is.null(theta)) stop("theta must be provided.")
    if (distribution == "beta" && is.null(USL) && is.null(LSL)) stop("USL or LSL must be provided.")
    if (!is.null(USL) && !is.null(LSL)) stop("Specify only one limit (USL or LSL), not both.")
    
    plan <- structure(list(n = n, k = k, sample_size = ceiling(n),
                           PRQ = PRQ, CRQ = CRQ, PR = alpha, CR = beta,
                           USL = USL, LSL = LSL,
                           sigma_type = sigma_type,
                           theta_type = theta_type,
                           method = method,
                           sigma = sigma, theta = theta,
                           distribution = distribution),
                      class = "VarPlan")
  }
  
  # final return plan
  return(plan)
}

test_that("manualPlan stores and applies unknown-theta Beta methods", {
  plans <- lapply(
    c("delta_mle", "delta_mom", "gk_adjustment"),
    function(method) manualPlan(
      distribution = "beta", n = 45, k = 2.2,
      theta = 300, theta_type = "unknown", USL = 0.05,
      method = method
    )
  )

  expect_identical(vapply(plans, `[[`, character(1), "method"),
                   c("delta_mle", "delta_mom", "gk_adjustment"))
  expect_true(all(vapply(plans, function(plan) {
    probability <- accProb(plan, 0.01)
    is.finite(probability) && probability >= 0 && probability <= 1
  }, logical(1))))
})

test_that("manualPlan defaults unknown-theta Beta plans to Delta-MLE", {
  plan <- manualPlan(
    distribution = "beta", n = 45, k = 2.2,
    theta = 300, theta_type = "unknown", USL = 0.05
  )
  expect_identical(plan$method, "delta_mle")
})

test_that("manualPlan rejects method outside unknown-theta Beta plans", {
  expect_error(
    manualPlan(distribution = "normal", n = 10, k = 1,
               method = "delta_mle"),
    "only applicable"
  )
  expect_error(
    manualPlan(distribution = "beta", n = 10, k = 1, theta = 100,
               USL = 0.1, method = "delta_mle"),
    "only applicable"
  )
  expect_error(
    manualPlan(distribution = "binomial", n = 10, c = 1,
               method = "delta_mle"),
    "only applicable"
  )
})

test_that("VarPlan print and summary report the unknown-theta method", {
  plan <- manualPlan(
    distribution = "beta", n = 45, k = 2.2,
    theta = 300, theta_type = "unknown", USL = 0.05,
    method = "delta_mom"
  )

  expect_output(print(plan), "Unknown-theta method: Delta-MoM", fixed = TRUE)
  expect_output(summary(plan), "Unknown-theta method: Delta-MoM", fixed = TRUE)
})

test_that("VarPlan output gives missing method metadata the new default label", {
  plan <- structure(
    list(
      distribution = "beta", n = 45, sample_size = 45, k = 2.2,
      theta = 300, theta_type = "unknown", USL = 0.05,
      PR = 0.05, CR = 0.10, PRQ = 0.01, CRQ = 0.05
    ),
    class = "VarPlan"
  )

  expect_output(print(plan), "Delta-MLE", fixed = TRUE)
  expect_output(summary(plan), "Delta-MLE", fixed = TRUE)
})

test_that("known-theta plan output does not report an unknown-theta method", {
  plan <- manualPlan(
    distribution = "beta", n = 20, k = 1.5,
    theta = 100, theta_type = "known", USL = 0.1
  )

  print_output <- capture.output(print(plan))
  summary_output <- capture.output(summary(plan))
  expect_false(any(grepl("Unknown-theta method", print_output, fixed = TRUE)))
  expect_false(any(grepl("Unknown-theta method", summary_output, fixed = TRUE)))
})

## -----------------------------------------------------------------------------
## accProb.R --- 
##
## Author: Ha Truong
##
## Created: 09 Mar 2025
##
## Purposes: Calculate acceptance probability
##
## Changelogs:
## -----------------------------------------------------------------------------

#' @export
accProb <- function(plan, p) {
  UseMethod("accProb")
}

#' @export
accProb.AttrPlan <- function(plan, p) {
  if (plan$distribution == "binomial") {
    pbinom(plan$c, plan$n, p)
  } else if (plan$distribution == "poisson") {
    ppois(plan$c, lambda = plan$n * p)
  } else {
    stop("Unknown distribution type: ", plan$distribution)
  }
}

#' @export
accProb.VarPlan <- function(plan, p) {
  if(plan$distribution == "normal") {
    n = plan$n
    k = plan$k
    
    if(plan$sigma_type == "unknown") {
      #Pa <- pnorm( sqrt(n/(1 + k^2/2)) * (qnorm(1-p) - k) )
      # This use t-distribution
      Pa <- 1- pt(k*sqrt(n), df=n-1, ncp=-qnorm(p)*sqrt(n))
    }
    else{
      Pa <- 1 - pnorm(sqrt(n) * (qnorm(p) + k))
    }
    return(round(Pa, 4))
  } else if (plan$distribution == "beta") {
    # Older objects may use m; current plan objects consistently use n.
    if (!is.null(plan[["m"]])) { m <- plan[["m"]] } else { m <- plan$n }
    k <- plan$k

    # This upper limit case
    if (!is.null(plan$USL)) {
      limtype <- "upper"
      limit <- plan$USL
    } else if (!is.null(plan$LSL)) { # This lower limit case
      limtype <- "lower"
      limit <- plan$LSL
    } else {
      limtype <- "upper" # set this as default
    }
    
    theta <- plan$theta
    mu <- muEst(p, USL = plan$USL, LSL = plan$LSL,
                theta = theta, dist = plan$distribution)

    if (identical(plan$theta_type, "unknown")) {
      # Missing metadata is interpreted as the new default so manually-created
      # and serialized plan objects follow the documented 0.0.9 behavior.
      method <- if (is.null(plan$method)) "delta_mle" else plan$method
      method <- match.arg(method, .beta_theta_methods)

      if (method != "gk_adjustment") {
        return(.beta_delta_acceptance_probability(
          mu = mu,
          theta = theta,
          n = m,
          k = k,
          limit = limit,
          limit_type = limtype,
          method = method
        ))
      }

      # Govindaraju--Kissling/package-style approximation: convert the supplied
      # unknown-theta sample size to its known-theta effective sample size.
      m <- m / (1 + 0.85 * k^2)
    }

    # Generate beta-distributed measurements
    a <- m * mu * theta
    b <- m * (1 - mu) * theta

    NSIM = 0
    if(NSIM > 0) # -------- This use simulation
    {
      ym <- rbeta(NSIM, a, b)
      var <- ym * (1 - ym) / theta # calculate variance VAR(Y)
      
      # Calculate acceptance criterion
      if (limtype == "lower") {
        acc_cri <- ym - k * sqrt(var)
        pa <- mean(acc_cri > limit)
      } else {
        acc_cri <- ym + k * sqrt(var)
        pa <- mean(acc_cri < limit)
      }
      return(pa)
    }
    else { #use closed-form
      # Compute quadratic coefficients
      A <- theta + k^2
      B <- -(2 * theta * limit + k^2)
      C <- theta * limit^2
      
      # Calculate discriminant
      discriminant <- (2 * theta * limit + k^2)^2 - 4 * A * C
      sqrt_disc <- sqrt(discriminant)
      
      # Compute roots
      z1 <- ( (2 * theta * limit + k^2) - sqrt_disc ) / (2 * A)
      z2 <- ( (2 * theta * limit + k^2) + sqrt_disc ) / (2 * A)
      
      # Determine Pa based on limit type
      if (limtype == "lower") {
        pa <- 1 - pbeta(z2, a, b)
      } else {
        pa <- pbeta(z1, a, b)
      }
      
      if (length(pa) != 1 || !is.numeric(pa) || is.na(pa)) {
        stop("No solution - Paccept can not be calculated base on input plan.")
      }
      return(pa)
    }
  } else {
    stop("Unknown distribution type: ", plan$distribution)
  }
}

## -----------------------------------------------------------------------------
## test-accProb.R
##
## Author: Ha Truong
## Created: 12 Nov 2025
## Purpose: Test accProb functions
## -----------------------------------------------------------------------------

# Attributes Sampling Plan
## -----------------------------------------------------------------------------
test_that("accProb.AttrPlan works for binomial", {
  plan <- list(
    distribution = "binomial",
    n = 20,
    c = 1
  )
  
  p <- 0.05
  expect_equal(
    accProb.AttrPlan(plan, p),
    pbinom(plan$c, plan$n, p)
  )
})

test_that("accProb.AttrPlan works for poisson", {
  plan <- list(
    distribution = "poisson",
    n = 20,
    c = 1
  )
  
  p <- 0.05
  expect_equal(
    accProb.AttrPlan(plan, p),
    ppois(plan$c, plan$n * p)
  )
})

test_that("accProb.AttrPlan errors on unknown distribution", {
  plan <- list(distribution = "xxx")
  expect_error(accProb.AttrPlan(plan, 0.1), "Unknown distribution")
})

# Attribute plans - extreme p
test_that("accProb.AttrPlan handles extreme p", {
  plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "binomial")
  
  expect_equal(accProb(plan, 0), 1)   # No defectives, always accept
  expect_equal(accProb(plan, 1), 0)   # All defective, never accept
  
  plan_pois <- optAttrPlan(PRQ = 0.01, CRQ = 0.05, distribution = "poisson")
  expect_equal(accProb(plan_pois, 0), 1)
})



# Variables Sampling Plan (Normal dist)
## -----------------------------------------------------------------------------

test_that("accProb.VarPlan works for normal (sigma known)", {
  plan <- list(
    distribution = "normal",
    n = 10,
    k = 1.5,
    sigma_type = "known"
  )
  
  p <- 0.05
  pa <- accProb.VarPlan(plan, p)
  
  expect_type(pa, "double")
  expect_true(pa >= 0 && pa <= 1)
})

test_that("accProb.VarPlan works for normal (sigma unknown)", {
  plan <- list(
    distribution = "normal",
    n = 10,
    k = 1.5,
    sigma_type = "unknown"
  )
  
  p <- 0.05
  pa <- accProb.VarPlan(plan, p)
  
  expect_type(pa, "double")
  expect_true(pa >= 0 && pa <= 1)
})


# Variables Sampling Plan (Beta dist)
## -----------------------------------------------------------------------------
test_that("accProb.VarPlan works for Beta upper limit", {
  plan <- list(
    distribution = "beta",
    m = 50,
    k = 1.5,
    theta = 100,
    theta_type = "unknown",
    USL = 0.10,
    n = 10
  )
  
  p <- 0.05
  pa <- accProb.VarPlan(plan, p)
  
  expect_type(pa, "double")
  expect_true(pa >= 0 && pa <= 1)
})

test_that("accProb.VarPlan works for Beta lower limit", {
  plan <- list(
    distribution = "beta",
    m = 50,
    k = 1.5,
    theta = 100,
    theta_type = "unknown",
    LSL = 0.10,
    n = 10
  )
  
  p <- 0.05
  pa <- accProb.VarPlan(plan, p)
  
  expect_type(pa, "double")
  expect_true(pa >= 0 && pa <= 1)
})

test_that("unknown-theta Beta acceptance probability dispatches by method", {
  base_plan <- list(
    distribution = "beta", n = 45, k = 2.2, theta = 300,
    theta_type = "unknown", USL = 0.05
  )

  probabilities <- vapply(
    c("delta_mle", "delta_mom", "gk_adjustment"),
    function(method) accProb.VarPlan(c(base_plan, list(method = method)), 0.01),
    numeric(1)
  )
  expect_true(all(is.finite(probabilities)))
  expect_true(all(probabilities >= 0 & probabilities <= 1))
  expect_equal(length(unique(round(probabilities, 8))), 3)
})

test_that("missing unknown-theta method uses Delta-MLE", {
  legacy_plan <- list(
    distribution = "beta", n = 45, k = 2.2, theta = 300,
    theta_type = "unknown", USL = 0.05
  )
  explicit_plan <- c(legacy_plan, list(method = "delta_mle"))

  expect_equal(
    accProb.VarPlan(legacy_plan, 0.01),
    accProb.VarPlan(explicit_plan, 0.01)
  )
})

test_that("Govindaraju-Kissling adjustment reproduces the 0.0.8 calculation", {
  plan <- list(
    distribution = "beta", n = 45, k = 2.2, theta = 300,
    theta_type = "unknown", method = "gk_adjustment", USL = 0.05
  )
  mu <- muEst(0.01, USL = plan$USL, theta = plan$theta, dist = "beta")
  effective_n <- plan$n / (1 + 0.85 * plan$k^2)
  shape1 <- effective_n * mu * plan$theta
  shape2 <- effective_n * (1 - mu) * plan$theta
  discriminant <- (2 * plan$theta * plan$USL + plan$k^2)^2 -
    4 * (plan$theta + plan$k^2) * plan$theta * plan$USL^2
  lower_root <- (2 * plan$theta * plan$USL + plan$k^2 -
    sqrt(discriminant)) / (2 * (plan$theta + plan$k^2))

  expect_equal(
    accProb.VarPlan(plan, 0.01),
    pbeta(lower_root, shape1, shape2),
    tolerance = 1e-12
  )
})

test_that("unknown-theta Beta acceptance rejects invalid method metadata", {
  plan <- list(
    distribution = "beta", n = 45, k = 2.2, theta = 300,
    theta_type = "unknown", method = "bad", USL = 0.05
  )
  expect_error(accProb.VarPlan(plan, 0.01), "arg.*one of")
})

test_that("accProb.VarPlan errors when no limit provided in Beta model", {
  plan <- list(
    distribution = "beta",
    m = 50,
    k = 1.5,
    theta = 100,
    n = 10
  )
  expect_error(accProb.VarPlan(plan, 0.05))
})

test_that("accProb.VarPlan errors on unknown distribution", {
  plan <- list(
    distribution = "xxx",
    n = 10
  )
  expect_error(accProb.VarPlan(plan, 0.1))
})

# Variable plans - edge limits
test_that("accProb.VarPlan works when pd at boundary", {
  plan_norm <- optVarPlan(PRQ = 0.01, CRQ = 0.05,
                          distribution = "normal", sigma_type = "known", sigma = 1)
  
  expect_true(accProb(plan_norm, 0) >= 0 & accProb(plan_norm, 0) <= 1)
  expect_true(accProb(plan_norm, 1) >= 0 & accProb(plan_norm, 1) <= 1)
})

---
title: "AccSamplingDesign: Acceptance Sampling Plan Design - R Package"
author: "Ha Truong"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    number_sections: true
vignette: >
  %\VignetteIndexEntry{AccSamplingDesign: Acceptance Sampling Plan Design - R Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(AccSamplingDesign)
library(VGAM)
```

# Introduction

The **AccSamplingDesign** package provides tools to create and evaluate 
acceptance sampling plans for both attribute and variable quality data. The 
focus is on controlling producer's and consumer's risk specifications while 
minimizing required sample sizes.

The package and its statistical foundations are described by Truong, Miranda,
and Kissling (2026), [*AccSamplingDesign: An R Package for Optimizing Acceptance
Sampling Plans*](https://doi.org/10.32614/RJ-2026-007), in *The R Journal*.

The package supports:

* Attributes sampling plans (e.g., pass/fail outcomes)
* Variables sampling plans under Normal and Beta distributions
* Easy-to-use functions for computing acceptance probability and visualizing OC curves

Install from CRAN:

```r
install.packages("AccSamplingDesign")
```

Or from GitHub:

```r
devtools::install_github("vietha/AccSamplingDesign")
```

# Attributes Sampling Examples

```{r}
# Create an attribute plan with binomial assumption
plan_attr <- optPlan(
  PRQ = 0.01,   # Acceptable quality level (1%)
  CRQ = 0.05,   # Rejectable quality level (5%)
  alpha = 0.02, # Producer's risk
  beta = 0.15,  # Consumer's risk
  distribution = "binomial"
)

# Summary of the plan
summary(plan_attr)

# Probability of accepting 3% defective
accProb(plan_attr, 0.03)

# Plot the OC curve
plot(plan_attr)
```

# Variables Sampling Examples 

## Normal Distribution, Known Sigma

```{r}
# Create a variable plan assuming known sigma
plan_var <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "known"
)

# Summary
summary(plan_var)

# Plot OC curve
plot(plan_var)
```

## Normal Distribution, Unknown Sigma

```{r}
# Create a variable plan assuming known sigma
plan_var2 <- optPlan(
  PRQ = 0.025,
  CRQ = 0.1,
  alpha = 0.05,
  beta = 0.10,
  distribution = "normal",
  sigma_type = "unknown"
)

# Summary
summary(plan_var2)
```

## Beta Distribution, Known Theta

```{r}
# Create a variable plan using Beta distribution
plan_beta <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "known",
  LSL = 0.00001          # Lower Specification Limit
)

# Summary
summary(plan_beta)

# Plot OC curve
plot(plan_beta)
# Plot OC curve be the process mean
plot(plan_beta, by = "mean")
```

## Beta Distribution, Unknown Theta

Three methods are available when theta is estimated during inspection:

* `"delta_mle"` uses analytical Fisher-information covariance and is the default;
* `"delta_mom"` uses analytical covariance propagated from Beta raw moments;
* `"gk_adjustment"` retains the earlier Govindaraju--Kissling sample-size adjustment.

The supplied `theta` is the planning/reference precision used to construct the
OC curve; `theta_type = "unknown"` means the inspection statistic estimates it.

```{r}
# Create a variable plan using Beta distribution
plan_beta2 <- optPlan(
  PRQ = 0.05,
  CRQ = 0.2,
  alpha = 0.05,
  beta = 0.10,
  distribution = "beta",
  theta = 44000000,
  theta_type = "unknown",
  method = "delta_mle",
  LSL = 0.00001
)

# Summary
summary(plan_beta2)
```

## Variables Sampling Plan Output Note

When designing a variables acceptance sampling plan using `optPlan()` or `optVarPlan()`,  
the returned object includes two related quantities for the sample size:

- **`sample_size`** — for practical application, the final plan uses the smallest integer greater than or equal to the theoretical value (`ceiling(n)`).
- **`n`** — the raw (non-rounded) computed sample size based on the optimization model.

Example:

```{r}
plan_beta2$sample_size  # integer sample size used in plan
plan_beta2$n            # raw computed (non-rounded) value
```

## Estimating θ for Beta-Based Plans
For Beta-distributed quality characteristics, the precision parameter **θ** 
reflects the concentration of the distribution around the mean.
If θ is unknown, it can be estimated from historical process data.

A practical approach is to use the `VGAM` package, which provides the `betaff` function for fitting a Beta distribution and obtaining θ:
  
```{r}
# Example data
y <- c(0.75, 0.68, 0.72, 0.70, 0.76)

# Fit Beta model
fit <- vglm(y ~ 1, betaff, data = data.frame(y = y))

# Extract estimated parameters
coef(fit, matrix = TRUE)
```

The output provides estimates of the mean (μ) and the precision parameter (θ).
Use the estimated θ directly in `optPlan()`, e.g.:

```{r}
theta_est <- exp(coef(fit, matrix = TRUE)[1, "loglink(phi)"])
theta_est
optPlan(PRQ = 0.05, CRQ = 0.20, alpha = 0.05, beta = 0.10,
           USL = 0.8, distribution = "beta", theta_type = "unknown",
           theta = theta_est)
```

## Optimization Approach for Sampling Plans

The **AccSamplingDesign** package determines optimal sample sizes and acceptance criteria for variable sampling plans via numerical optimization.

- **Normal Plans (σ unknown):** Optimization is performed using the derivative-free method of Nelder and Mead (1965) via the `optim()` function in R. This method is robust for the non-linear objective functions that arise when σ is unknown.

- **Beta Plans:** Optimization is performed using the `optim()` function in R with the `"L-BFGS-B"` method (Byrd et al., 1995), which efficiently handles bound constraints on sample size and acceptance numbers.

This approach ensures stable and efficient plan calculations while relying solely on base R functionality, avoiding additional package dependencies.


# Custom Plan Comparison

```{r}
# Define range of defect rates
pd <- seq(0, 0.15, by = 0.001)

# Generate OC data from optimal plan
oc_opt <- OCdata(plan = plan_attr, pd = pd)

# Compare with manual plans
mplan1 <- manualPlan(n = plan_attr$n, c = plan_attr$c - 1, distribution = "binomial")
oc_alt1 <- OCdata(plan = mplan1, pd = pd)

# Plot comparison
plot(pd, oc_opt$paccept, type = "l", col = "blue", lwd = 2,
     xlab = "Proportion Defective", ylab = "Probability of Acceptance",
     main = "OC Curves Comparison for Attributes Sampling Plan")
lines(pd, oc_alt1$paccept, col = "red", lwd = 2, lty = 2)
legend("topright", legend = c("Optimal Plan", "Manual Plan c - 1"),
       col = c("blue", "red"), lty = c(1, 2), lwd = 2)
```


# References
This vignette provides a quick start for using the **AccSamplingDesign** package. 
For a full discussion of the statistical foundations, models, and optimization 
methods used, please refer to the foundation sources such as:

* Schilling, E.G., & Neubauer, D.V. (2017). Acceptance Sampling in Quality Control (3rd ed.). CRC Press.

* Wilrich, P.T. (2004). Single Sampling Plans for Inspection by Variables under a Variance Component Situation. In Frontiers in Statistical Quality Control 7.

* Truong, H., Miranda, V., & Kissling, R. (2026). AccSamplingDesign: An R Package for Optimizing Acceptance Sampling Plans. *The R Journal*, 18(1), 368--381. https://doi.org/10.32614/RJ-2026-007

* Govindaraju, K., & Kissling, R. (2015). Sampling plans for Beta-distributed compositional fractions. Quality Engineering, 27(1), 1–13.

* J. A. Nelder and R. Mead. A simplex method for function minimization. The Computer Journal, 7(4): 308–313, 1965. DOI 10.1093/comjnl/7.4.308.

* R. H. Byrd, P. Lu, J. Nocedal and C. Zhu. A limited memory algorithm for bound constrained optimization. SIAM Journal on Scientific Computing, 16(5): 1190–1208, 1995. DOI 10.1137/0916069.

\name{OCdata}
\alias{OCdata}
\alias{OCdata.AttrPlan}
\alias{OCdata.VarPlan}
\title{Generic function for OC Curve Generation}
\description{
  Generic function to compute Operating Characteristic (OC) curve data
  from an acceptance sampling plan.
}
\usage{
OCdata(plan, pd = NULL)
}
\arguments{
  \item{plan}{An object of class \code{AttrPlan} or \code{VarPlan}.}
  \item{pd}{Vector of quality levels (proportions of nonconforming items).}
}
\details{
  This is a generic function. Methods are defined for objects of class
  \code{AttrPlan} and \code{VarPlan}, which compute the probability of
  acceptance across a range of quality levels (proportions of nonconforming).

  See \code{\link{OCdata.AttrPlan}} and \code{\link{OCdata.VarPlan}} for details.
}
\value{
  An object of class \code{"OCdata"}, a list containing:
  \itemize{
    \item \code{pd} — quality levels (e.g. proportion defective)
    \item \code{paccept} — probability of acceptance at each level
    \item \code{process_means} — process means, if applicable
    \item \code{dist}, \code{n}, \code{k}, \code{c} — plan parameters
  }
}
\seealso{
  \code{\link{optPlan}}, \code{\link{manualPlan}}
}
\author{
  Ha Truong
}

\name{optVarPlan}
\alias{optVarPlan}
\title{Variables Acceptance Sampling Plan}
\description{
  Creates Variable Acceptance Sampling plans for normal or beta distributed measurements.
}
\usage{
optVarPlan(PRQ, CRQ, alpha = 0.05, beta = 0.10, USL = NULL, LSL = NULL,
           distribution = c("normal", "beta"), sigma_type = c("known", "unknown"),
           theta_type = c("known", "unknown"), sigma = NULL, theta = NULL,
           method = c("delta_mle", "delta_mom", "gk_adjustment"))
}
\arguments{
  \item{PRQ}{Producer Risk Quality (must be within valid range for the chosen distribution).}
  
  \item{CRQ}{Consumer Risk Quality (must be greater than PRQ and within valid range).}
  
  \item{alpha}{Producer's risk (numeric between 0 and 1).}
  
  \item{beta}{Consumer's risk (numeric between 0 and 1).}
  
  \item{USL}{Upper Specification Limit (numeric). Only one of \code{USL} or \code{LSL} should be provided.}
  
  \item{LSL}{Lower Specification Limit (numeric). Only one of \code{USL} or \code{LSL} should be provided.}
  
  \item{distribution}{Measurement distribution: \code{"normal"} or \code{"beta"}.}
  
  \item{sigma_type}{Indicates whether sigma (population standard deviation) is \code{"known"} or \code{"unknown"}.}
  
  \item{theta_type}{Indicates whether theta (population precision parameter for beta) is \code{"known"} or \code{"unknown"}.}
  
  \item{sigma}{Known standard deviation (used for normal distribution). Required if \code{sigma_type = "known"}.}
  
  \item{theta}{Precision (dispersion) parameter for the beta distribution.
  If unknown, it can be estimated from historical data using functions such as \code{\link[VGAM]{betaff}}.}

  \item{method}{Unknown-theta Beta method: \code{"delta_mle"} (default),
  \code{"delta_mom"}, or \code{"gk_adjustment"}. It is only applicable when
  \code{distribution = "beta"} and \code{theta_type = "unknown"}.}
}
\details{
  The function generates variable acceptance sampling plans based on specified producer and consumer risks and either a normal or beta distribution model.

  The specification limit must be defined via either \code{USL} (upper specification limit) or \code{LSL} (lower specification limit), depending on whether the one-sided quality criterion concerns the upper or lower tail. Only one limit should be provided.

  For the \code{"normal"} model, the plan can be computed assuming the standard deviation 
  (\code{sigma}) is either known or estimated from data. Optimization for unknown \code{sigma} 
  is performed using the derivative-free Nelder-Mead method (Nelder and Mead, 1965) via 
  \code{\link[stats]{optim}} in base R.
  
  For the \code{"beta"} model, the dispersion (precision) parameter \code{theta} determines 
  how concentrated the distribution is around the mean. Users must either:
  \itemize{
    \item provide a known value for \code{theta}, typically obtained from prior process studies or historical data; or
    \item estimate it from sample data using a fitting function such as \code{\link[VGAM]{betaff}} in the \pkg{VGAM} package.
  }
  Optimization for Beta plans is performed using \code{\link[stats]{optim}} with the \code{"L-BFGS-B"} method 
  (Byrd et al., 1995) to handle bounds on sample size and acceptance numbers.

  For unknown precision, Delta--MLE uses inverse expected Fisher information
  and Delta--MoM uses analytical covariance propagated from the first four Beta
  raw moments. The Govindaraju--Kissling adjustment retains the package's
  earlier sample-size approximation.
  
  This approach ensures stable and efficient plan calculation while relying solely on base R functionality.
}

\value{
  A \code{VarPlan} object containing:
  \item{distribution}{Distribution used ("normal" or "beta").}
  \item{sample_size}{Final sample size after rounding up to the next integer, for practical application.}
  \item{k}{Acceptability constant.}
  \item{n}{Unrounded sample size.}
}
\references{
  * H. Truong, V. Miranda and R. Kissling (2026).
  AccSamplingDesign: An R Package for Optimizing Acceptance Sampling Plans.
  \emph{The R Journal}, 18(1), 368--381.
  \doi{10.32614/RJ-2026-007}.

  * ISO 3951-1:2013 - Sampling procedures for inspection by variables.
  
  * Wilrich, PT. (2004). Single Sampling Plans for Inspection by Variables under a Variance Component Situation. In: Lenz, HJ., Wilrich, PT. (eds) Frontiers in Statistical Quality Control 7. Physica, Heidelberg.
  
  * K. Govindaraju and R. Kissling (2015). Sampling plans for Beta-distributed compositional fractions.
  
  * J. A. Nelder and R. Mead. A simplex method for function minimization. The Computer Journal, 7(4): 308–313, 1965. DOI 10.1093/comjnl/7.4.308.
  
  * R. H. Byrd, P. Lu, J. Nocedal and C. Zhu. A limited memory algorithm for bound constrained optimization. SIAM Journal on Scientific Computing, 16(5): 1190–1208, 1995. DOI 10.1137/0916069.
}
\author{
Ha Truong
}
\examples{
# Example for normal distribution plan
norm_plan <- optVarPlan(
  PRQ = 0.025,        # Acceptable quality level (% nonconforming)
  CRQ = 0.1,          # Rejectable quality level (% nonconforming)
  alpha = 0.05,       # Producer's risk
  beta = 0.1,         # Consumer's risk
  distribution = "normal",
  USL = 10
)
summary(norm_plan)

# Example for beta distribution plan
beta_plan <- optVarPlan(
  PRQ = 0.025,        # Target quality level (% nonconforming)
  CRQ = 0.1,          # Minimum quality level (% nonconforming)
  alpha = 0.05,       # Producer's risk
  beta = 0.1,         # Consumer's risk
  distribution = "beta",
  theta = 44000000,   # Beta distribution parameter
  LSL = 0.00001
)
summary(beta_plan)
}

\name{manualPlan}
\alias{manualPlan}
\title{Create manual acceptance sampling (AS) plan from User Inputs}
\description{
  Constructs an \code{AttrPlan} or \code{VarPlan} object from the given parameters.
}
\usage{
manualPlan(distribution = c("binomial", "poisson", "normal", "beta"),
           n = NULL, c = NULL, k = NULL,
           USL = NULL, LSL = NULL, sigma = NULL, theta = NULL,
           sigma_type = c("known", "unknown"),
           theta_type = c("known", "unknown"),
           method = c("delta_mle", "delta_mom", "gk_adjustment"))
}
\arguments{
  \item{distribution}{One of \code{"binomial"}, \code{"poisson"}, \code{"normal"}, or \code{"beta"}.}

  \item{n}{Sample size.}

  \item{c}{Acceptance number (for attribute sampling).}

  \item{k}{Acceptability constant (for variable sampling).}

  \item{USL}{Upper specification limit.}

  \item{LSL}{Lower specification limit.}

  \item{sigma}{Standard deviation (for normal plans).}

  \item{theta}{Precision parameter (for beta plans).}

  \item{sigma_type}{Either \code{"known"} or \code{"unknown"} (for normal).}

  \item{theta_type}{Either \code{"known"} or \code{"unknown"} (for beta).}

  \item{method}{Method used by an unknown-theta Beta plan. Delta--MLE is the
  default; alternatives are \code{"delta_mom"} and \code{"gk_adjustment"}.}
}
\details{
  This function provides a user-friendly wrapper to construct AS plan directly
  from parameters.
  Internally, it constructs the appropriate \code{AttrPlan} or \code{VarPlan},
  from given paramenters.
}
\value{
  An object of class \code{"AttrPlan"} or \code{VarPlan}.
}
\seealso{
  \code{\link{optPlan}}, \code{\link{OCdata}}
}
\examples{
# Attribute sampling with user-defined parameters
plan1 <- manualPlan(n = 100, c = 2, distribution = "binomial")

# Variable sampling (normal)
plan2 <- manualPlan(n = 30, k = 1.5, distribution = "normal", USL = 10, sigma = 1)

# Unknown-theta Beta plan using analytical Delta--MoM
plan3 <- manualPlan(n = 45, k = 2.2, distribution = "beta",
                    theta = 300, theta_type = "unknown", method = "delta_mom",
                    USL = 0.05)

}
\author{
  Ha Truong
}

\name{plot.AttrPlan}
\alias{plot.AttrPlan}
\title{Plot the OC Curve for Attribute Sampling Plans}
\description{
  Plots the Operating Characteristic (OC) curve for an attribute sampling plan object of class \code{AttrPlan}.
}
\usage{
\method{plot}{AttrPlan}(x, pd = NULL, ...)
}
\arguments{
  \item{x}{An object of class \code{AttrPlan} representing an attribute acceptance sampling plan.}

  \item{pd}{Optional vector of proportions of nonconforming items. If \code{NULL} (default), a range is automatically generated.}

  \item{...}{Additional graphical parameters passed to \code{plot()}.}
}
\details{
  This method computes and visualizes the probability of acceptance (\code{P(accept)}) as a function of the proportion of nonconforming items in the population, based on the attribute sampling plan.

  The plot also includes reference lines at the plan's producer and consumer quality levels (\code{PRQ}, \code{CRQ}) and their corresponding acceptance probabilities.
}
\value{
  A plot showing the OC curve for the given attribute sampling plan.
}
\seealso{
  \code{\link{optAttrPlan}}, \code{\link{accProb}}, \code{\link{OCdata}}
}
\examples{
# Create attribute plan
plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.1)

# Plot OC curve
plot(plan)

# With custom pd
plot(plan, pd = seq(0, 0.15, by = 0.001))
}
\author{Ha Truong}

\name{summary.VarPlan}
\alias{summary.VarPlan}
\title{Summarize Variables Acceptance Plan}
\description{
Detailed summaries for Variables Acceptance Sampling plans.
}
\usage{
\method{summary}{VarPlan}(object, ...)
}
\arguments{
  \item{object}{Plan object to summarize}
  \item{...}{Additional parameters (ignored)}
}

\author{
Ha Truong
}

\value{
No return value. This function is called for its side effect of printing a 
formatted summary of the variable sampling plan to the console.
}

\examples{
var_plan <- optVarPlan(
  PRQ = 0.025,       # Acceptable quality level (% nonconforming)
  CRQ = 0.1,         # Rejectable quality level (% nonconforming)
  alpha = 0.05,      # Producer's risk
  beta = 0.1,        # Consumer's risk
  distribution = "normal"
)
summary(var_plan)
}

\name{muEst}
\alias{muEst}
\title{Estimate Mean \eqn{\mu}{mu} Based on Specification Limits and Probability}
\description{
  Computes the estimated mean \eqn{\mu}{mu} for a given level of quality and specification limit under either a normal or beta distribution.
}
\usage{
muEst(p, USL = NULL, LSL = NULL, 
       sigma = NULL, theta = NULL, 
       dist = c("normal", "beta"))
}
\arguments{
  \item{p}{Level of quality (numeric, between 0 and 1).}

  \item{USL}{Upper specification limit (numeric). Only one of \code{USL} or \code{LSL} should be provided.}

  \item{LSL}{Lower specification limit (numeric). Only one of \code{USL} or \code{LSL} should be provided.}

  \item{sigma}{Standard deviation (numeric) for the normal distribution. Must be provided if \code{dist = "normal"}.}

  \item{theta}{Theta parameter (numeric) for the beta distribution. Must be provided if \code{dist = "beta"}.}

  \item{dist}{Distribution type. Either \code{"normal"} or \code{"beta"}.}
}
\details{
  The function estimates the mean \eqn{\mu}{mu} corresponding to a given tail probability \code{p}, assuming that the process output follows either a normal or beta distribution, and that the probability of being beyond the provided specification limit equals \code{1 - p}.

  \itemize{
    \item For the normal distribution, the mean is calculated using the inverse cumulative distribution function (quantile function) of the normal distribution.
    \item For the beta distribution, the mean is solved numerically such that the CDF at the specified limit equals \code{p}, given the shape determined by \code{theta}.
  }

  Exactly one of \code{USL} or \code{LSL} must be provided to define whether the probability refers to the upper or lower tail.
}
\value{
  Returns the estimated mean \eqn{\mu}{mu} as a numeric value.
}
\author{
Ha Truong
}
\examples{
  # Example for normal distribution with lower specification limit (LSL)
  muEst(p = 0.95, LSL = 10, sigma = 2, dist = "normal")

  # Example for beta distribution with upper specification limit (USL)
  muEst(p = 0.95, USL = 0.7, theta = 500, dist = "beta")
}

\name{plot.OCdata}
\alias{plot.OCdata}
\title{Plot Method for OCdata Objects}
\description{
  Plots the Operating Characteristic (OC) curve from an object of class \code{"OCdata"}, either by proportion nonconforming or process mean levels.
}
\usage{
\method{plot}{OCdata}(x, by = c("pd", "mean"), ...)
}
\arguments{
  \item{x}{An object of class \code{"OCdata"}, typically generated using \code{OCdata()}.}

  \item{by}{A character string indicating the type of OC curve to plot. Options are:
    \describe{
      \item{\code{"pd"}}{(Default) Plot the OC curve by proportion nonconforming.}
      \item{\code{"mean"}}{Plot the OC curve by estimated process mean levels (only available for variable sampling plans).}
    }
  }

  \item{...}{Additional graphical parameters passed to the \code{plot()} function.}
}
\details{
  This method visualizes the OC curve based on the content of the \code{"OCdata"} object.

  By default, the curve is plotted against the proportion of nonconforming items (\code{@pd}). If \code{by = "mean"} is specified and the plan includes valid mean-level estimates (\code{@process_means}), the curve is plotted against mean levels.

  If \code{by = "mean"} is requested but no mean estimates are available (e.g., for attribute plans), a message will be shown and no plot will be drawn.
}

\value{
  A plot showing the OC curve for the given attribute/variable sampling plan.
}

\examples{
# Attribute plan
plan_attr <- optAttrPlan(PRQ = 0.01, CRQ = 0.05)
oc_attr <- OCdata(plan_attr)
plot(oc_attr)               # OC curve by pd (default)
plot(oc_attr, by = "mean")  # Will show message if not available

# Variable plan
plan_var <- optVarPlan(PRQ = 0.025, CRQ = 0.1, USL = 0.1,
                       distribution = "normal", sigma=0.01)
oc_var <- OCdata(plan_var)
plot(oc_var)                # OC curve by pd
plot(oc_var, by = "mean")   # OC curve by mean levels
}
\seealso{
  \code{\link{OCdata}}, \code{\link{optAttrPlan}}, \code{\link{optVarPlan}}
}
\author{
  Ha Truong
}

\name{optPlan}
\alias{optPlan}
\title{Optimal Acceptance Sampling Plan}
\description{
Design optimal variable acceptance sampling plans based on specified parameters. 
Supports different distributions (binomial, poisson, normal, beta) and accommodates known 
or unknown standard deviation and process parameters.
}
\usage{
optPlan(PRQ, CRQ, alpha = 0.05, beta = 0.10, USL = NULL, LSL = NULL,
        distribution = c("binomial", "poisson", "normal", "beta"),
        sigma_type = c("known", "unknown"),
        theta_type = c("known", "unknown"),
        sigma = NULL, theta = NULL,
        method = c("delta_mle", "delta_mom", "gk_adjustment"))
}
\arguments{
  \item{PRQ}{Producer's risk quality level (e.g., acceptable quality level).}
  \item{CRQ}{Consumer's risk quality level (e.g., rejectable quality level).}
  \item{alpha}{Producer's risk (Type I error), default is 0.05.}
  \item{beta}{Consumer's risk (Type II error), default is 0.10.}
  \item{USL}{Upper Specification Limit. Required for variable sampling plans.}
  \item{LSL}{Lower Specification Limit. Required for variable sampling plans.}
  \item{distribution}{Distribution type used in the plan. Can be \code{"binomial"}, \code{"poisson"}, \code{"normal"}, or \code{"beta"}.}
  \item{sigma_type}{Indicates if the standard deviation (\code{sigma}) is known or unknown.}
  \item{theta_type}{Indicates if the process parameter (\code{theta}) is known or unknown.}
  \item{sigma}{Known standard deviation of the process, if applicable.}
  \item{theta}{Precision (dispersion) parameter for the beta distribution.
  If unknown, it can be estimated from historical data using functions such as \code{\link[VGAM]{betaff}}.}
  \item{method}{Method for a Beta plan with \code{theta_type = "unknown"}:
  \code{"delta_mle"} (default), \code{"delta_mom"}, or
  \code{"gk_adjustment"}. It must not be supplied in other cases.}
}
\details{
This function designs optimal acceptance sampling plans by balancing producer's 
and consumer's risks under specified quality levels. It supports plans for 
attributes (binomial) and variables (normal or beta distributions), including 
cases with unknown standard deviation or distributional parameters.

For the \code{"normal"} model, the plan can be computed assuming the standard deviation 
(\code{sigma}) is either known or estimated from data. Optimization for unknown \code{sigma} 
is performed using the derivative-free Nelder-Mead method (Nelder and Mead, 1965) via 
\code{\link[stats]{optim}} in base R.

For the \code{"beta"} model, the dispersion (precision) parameter \code{theta} determines 
how concentrated the distribution is around the mean. Users must either:
\itemize{
  \item provide a known value for \code{theta}, typically obtained from prior process studies or historical data; or
  \item estimate it from sample data using a fitting function such as \code{\link[VGAM]{betaff}} in the \pkg{VGAM} package.
}
Optimization for Beta plans is performed using \code{\link[stats]{optim}} with the \code{"L-BFGS-B"} method 
(Byrd et al., 1995) to handle bounds on sample size and acceptance numbers.

For unknown \code{theta}, the Delta methods approximate the joint uncertainty
of the estimated Beta mean and precision. Delta--MLE uses inverse expected
Fisher information; Delta--MoM propagates the first four Beta raw moments.
The \code{"gk_adjustment"} option preserves the earlier
Govindaraju--Kissling/package-style sample-size adjustment.

This approach ensures stable and efficient plan calculation while relying solely on base R functionality.
}

\value{
Returns a list or data frame with optimal sample size(s) and critical value(s) 
based on the specified parameters and distribution.
  \item{distribution}{Selected distribution}
  \item{sample_size}{Final sample size after rounding up to the next integer, for practical application.}
  \item{n}{Sample size (unrounded for Variables Acceptance Sampling).}
  \item{k}{Acceptability constant - For Variables Acceptance Sampling.}
  \item{c}{Acceptance number - For Attributes Acceptance Sampling.}
}
\references{
  * H. Truong, V. Miranda and R. Kissling (2026).
  AccSamplingDesign: An R Package for Optimizing Acceptance Sampling Plans.
  \emph{The R Journal}, 18(1), 368--381.
  \doi{10.32614/RJ-2026-007}.

  * ISO 2859-1:1999 - Sampling procedures for inspection by attributes 
  
  * ISO 3951-1:2013 - Sampling procedures for inspection by variables.
  
  * Wilrich, PT. (2004). Single Sampling Plans for Inspection by Variables under a Variance Component Situation. In: Lenz, HJ., Wilrich, PT. (eds) Frontiers in Statistical Quality Control 7. Physica, Heidelberg.
  
  * K. Govindaraju and R. Kissling (2015). Sampling plans for Beta-distributed compositional fractions.
  
  * J. A. Nelder and R. Mead. A simplex method for function minimization. The Computer Journal, 7(4): 308–313, 1965. DOI 10.1093/comjnl/7.4.308.
  
  * R. H. Byrd, P. Lu, J. Nocedal and C. Zhu. A limited memory algorithm for bound constrained optimization. SIAM Journal on Scientific Computing, 16(5): 1190–1208, 1995. DOI 10.1137/0916069.
}
\author{
Ha Truong
}
\examples{
# Example usage (normal distribution, known sigma):
optPlan(PRQ = 0.005, CRQ = 0.03, alpha = 0.05, beta = 0.10, 
        distribution = "normal", sigma_type = "known")

# Example usage (beta distribution, unknown theta):
optPlan(PRQ = 0.025, CRQ = 0.10, alpha = 0.05, beta = 0.10, 
        distribution = "beta", theta = 6.6e8, 
        theta_type = "unknown", method = "delta_mle", LSL = 5.65e-6)
}

\name{accProb}
\alias{accProb}
\title{Acceptance Probability}
\description{
  Calculate the probability of acceptance for a given quality level.
}
\usage{
accProb(plan, p)
}
\arguments{
  \item{plan}{Acceptance plan object (\code{AttrPlan}/\code{VarPlan}).}
  \item{p}{True quality level (proportion of nonconforming).}
}
\value{
  Numeric probability between 0 and 1.
}
\details{
For Beta plans with unknown theta, the calculation follows the \code{method}
stored in the plan: analytical Delta--MLE, analytical Delta--MoM, or the
Govindaraju--Kissling sample-size adjustment.
}

\author{
Ha Truong
}

\examples{
# Example for attribute plan
attr_plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.1)
accProb(attr_plan, 0.05)

# Example for variable plan (normal distribution)
var_plan <- optVarPlan(
  PRQ = 0.025,        # Acceptable quality level (% nonconforming)
  CRQ = 0.1,         # Rejectable quality level (% nonconforming)
  alpha = 0.05,      # Producer's risk
  beta = 0.1,        # Consumer's risk
  distribution = "normal"
)
accProb(var_plan, 0.05)
}

\name{optAttrPlan}
\alias{optAttrPlan}
\title{Attributes Acceptance Sampling Plan}
\description{
Designs Attributes Acceptance Sampling plans using producer/consumer risk criteria.
}
\usage{
optAttrPlan(PRQ, CRQ, alpha = 0.05, beta = 0.10, 
            distribution = c("binomial", "poisson"))
}
\arguments{
  \item{PRQ}{Producer Risk Quality (0 < PRQ < 1)}
  \item{CRQ}{Consumer Risk Quality (PRQ < CRQ < 1)}
  \item{alpha}{Producer's risk (0.05 default)}
  \item{beta}{Consumer's risk (0.10 default)}
  \item{distribution}{Support binomial and poisson distribution}
}
\value{
AttrPlan object containing:
\item{n}{Sample size}
\item{c}{Acceptance number}
\item{PRQ}{Input PRQ value}
\item{CRQ}{Input CRQ value}
\item{distribution}{Selected distribution}
}

\references{
ISO 2859-1:1999 - Sampling procedures for inspection by attributes 

Schilling, E.G., & Neubauer, D.V. (2017). Acceptance Sampling in Quality Control (3rd ed.). Chapman and Hall/CRC. https://doi.org/10.4324/9781315120744
}
\author{
Ha Truong
}


\examples{
plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.1, alpha = 0.05, beta = 0.1, 
                    distribution = "binomial")
}

\name{plot.VarPlan}
\alias{plot.VarPlan}
\title{Plot the OC Curve for Variable Sampling Plans}
\description{
  Plots the Operating Characteristic (OC) curve for an object of class \code{VarPlan}. Supports plotting against either the proportion of nonconforming items or the corresponding process mean levels, depending on availability.
}
\usage{
\method{plot}{VarPlan}(x, pd = NULL, by = c("pd", "mean"), ...)
}
\arguments{
  \item{x}{An object of class \code{VarPlan} representing a variable acceptance sampling plan.}

  \item{pd}{Optional numeric vector of proportions of nonconforming items to evaluate. If \code{NULL} (default), a suitable range is generated automatically.}

  \item{by}{Character string indicating which x-axis to use for plotting. Either \code{"pd"} for proportion nonconforming (default) or \code{"mean"} for process mean levels. If \code{"mean"} is selected but the plan lacks specification limits, an error is raised.}

  \item{...}{Additional graphical parameters passed to \code{plot()}.}
}
\details{
  This plotting method visualizes the probability of acceptance (\code{P(accept)}) against the desired metric, based on the parameters of a variable sampling plan.

  If \code{by = "pd"}, the x-axis represents the proportion of nonconforming items. If \code{by = "mean"} and the plan defines \code{limit_type} and \code{spec_limit}, the function estimates corresponding process means using \code{\link{muEst}} and plots the OC curve by those mean values.

  Reference lines for the Producer's Risk Quality (PRQ) and Consumer's Risk Quality (CRQ), along with their respective acceptance probabilities, are shown when plotting by proportion.
}
\value{
  A plot showing the OC curve for the given variable sampling plan, either by nonconforming proportion or mean level.
}
\seealso{
  \code{\link{optVarPlan}}, \code{\link{accProb}}, \code{\link{muEst}}, \code{\link{OCdata}}, \code{\link{plot.OCdata}}
}
\examples{
# Variable sampling plan with specification limits
plan <- optVarPlan(
  PRQ = 0.025, CRQ = 0.1,
  alpha = 0.05, beta = 0.1,
  distribution = "normal",
  USL = 3, sigma = 0.1
)

# Plot by proportion nonconforming
plot(plan, by = "pd")

# Plot by estimated mean level (requires spec_limit and limit_type)
plot(plan, by = "mean")

# Custom pd vector
plot(plan, pd = seq(0.01, 0.15, by = 0.001))
}
\author{Ha Truong}

\name{summary.AttrPlan}
\alias{summary.AttrPlan}
\title{Summarize Attribute Acceptance Plan}
\description{
Detailed summaries for attribute acceptance plans.
}
\usage{
\method{summary}{AttrPlan}(object, ...)
}
\arguments{
  \item{object}{Plan object to summarize}
  \item{...}{Additional parameters (ignored)}
}

\author{
Ha Truong
}

\value{
No return value. This function is called for its side effect of printing a 
formatted summary of the attribute sampling plan to the console.
}

\examples{
attr_plan <- optAttrPlan(PRQ = 0.01, CRQ = 0.1)
summary(attr_plan)
}
